/*
 * Copyright 2010 Alibaba Group Holding Limited.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.citrus.service.requestcontext;

import static com.alibaba.citrus.util.Assert.*;
import static com.alibaba.citrus.util.StringUtil.*;

/**
 * һ<code>RequestContext</code>صϢ
 * 
 * @author Michael Zhou
 */
public interface RequestContextInfo<R extends RequestContext> {
    /**
     * ȡõǰfactoryɵrequest contextӿڡ
     */
    Class<R> getRequestContextInterface();

    /**
     * ȡproxyĽӿڡ
     */
    Class<? extends R> getRequestContextProxyInterface();

    /**
     * ȡõǰfactoryɵrequest contextṩfeatures
     * νfeaturesһЩַRequestContextChainingЩfeatureϵrequest
     * context
     */
    String[] getFeatures();

    /**
     * ָǰrequest contextЩfeatures֮ǰ֮
     */
    FeatureOrder[] featureOrders();

    /**
     * request context feature˳
     */
    abstract class FeatureOrder {
        public final String feature;

        public FeatureOrder(String feature) {
            this.feature = assertNotNull(trimToNull(feature), "feature");
        }
    }

    /**
     * ʾǰrequest contextӦṩָfeaturerequest context֮ǰ
     */
    class BeforeFeature extends FeatureOrder {
        public BeforeFeature(String feature) {
            super(feature);
        }

        @Override
        public String toString() {
            return "Before " + feature;
        }
    }

    /**
     * ʾǰrequest contextӦṩָfeaturerequest context֮
     */
    class AfterFeature extends FeatureOrder {
        public AfterFeature(String feature) {
            super(feature);
        }

        @Override
        public String toString() {
            return "After " + feature;
        }
    }

    /**
     * ʾǰrequest contextǰṩָfeaturerequest contextڡ
     */
    class RequiresFeature extends AfterFeature {
        public RequiresFeature(String feature) {
            super(feature);
        }

        @Override
        public String toString() {
            return "Requires " + feature;
        }
    }
}
