/*
 * Copyright 2010 Alibaba Group Holding Limited.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.citrus.service.requestcontext.parser;

import java.io.UnsupportedEncodingException;
import java.text.DateFormat;
import java.util.Date;
import java.util.Set;

import org.springframework.beans.TypeConverter;
import org.springframework.core.MethodParameter;

/**
 * һȡHTTPеĲcookies
 * <p>
 * ע⣺cookieƿܱתȫдȫСд ǸļеĲ<code>url.case.folding</code>
 * ָġ
 * </p>
 * 
 * @author Michael Zhou
 */
public interface ValueParser {
    /**
     * ȡת
     */
    TypeConverter getTypeConverter();

    // =============================================================
    //  ѯķ
    // =============================================================

    /**
     * ȡֵ
     * 
     * @return ֵ
     */
    int size();

    /**
     * жǷֵ
     * 
     * @return ֵ򷵻<code>true</code>
     */
    boolean isEmpty();

    /**
     * ǷָƵĲ
     * 
     * @param key ҪҵĲ
     * @return ڣ򷵻<code>true</code>
     */
    boolean containsKey(String key);

    /*
     * ȡвļϡ
     * @return вļ
     */
    Set<String> keySet();

    /*
     * ȡв顣
     * @return в
     */
    String[] getKeys();

    // =============================================================
    //  ȡòֵ
    // =============================================================

    /**
     * ȡòֵָƵĲڣ򷵻<code>false</code>
     * 
     * @param key 
     * @return ֵ
     */
    boolean getBoolean(String key);

    /**
     * ȡòֵָƵĲڣ򷵻ָĬֵ
     * 
     * @param key 
     * @param defaultValue Ĭֵ
     * @return ֵ
     */
    boolean getBoolean(String key, boolean defaultValue);

    /**
     * ȡòֵָƵĲڣ򷵻<code>0</code>
     * 
     * @param key 
     * @return ֵ
     */
    byte getByte(String key);

    /**
     * ȡòֵָƵĲڣ򷵻ָĬֵ
     * 
     * @param key 
     * @param defaultValue Ĭֵ
     * @return ֵ
     */
    byte getByte(String key, byte defaultValue);

    /**
     * ȡָֽڡֽǸ<code>getCharacterEncoding()</code>صַбġ
     * 
     * @param key 
     * @return ֵֽ
     * @throws UnsupportedEncodingException ָ˴ıַ
     */
    byte[] getBytes(String key) throws UnsupportedEncodingException;

    /**
     * ȡòֵָƵĲڣ򷵻<code>'\0'</code>
     * 
     * @param key 
     * @return ֵ
     */
    char getChar(String key);

    /**
     * ȡòֵָƵĲڣ򷵻ָĬֵ
     * 
     * @param key 
     * @param defaultValue Ĭֵ
     * @return ֵ
     */
    char getChar(String key, char defaultValue);

    /**
     * ȡòֵָƵĲڣ򷵻<code>0</code>
     * 
     * @param key 
     * @return ֵ
     */
    double getDouble(String key);

    /**
     * ȡòֵָƵĲڣ򷵻ָĬֵ
     * 
     * @param key 
     * @param defaultValue Ĭֵ
     * @return ֵ
     */
    double getDouble(String key, double defaultValue);

    /**
     * ȡòֵָƵĲڣ򷵻<code>0</code>
     * 
     * @param key 
     * @return ֵ
     */
    float getFloat(String key);

    /**
     * ȡòֵָƵĲڣ򷵻ָĬֵ
     * 
     * @param key 
     * @param defaultValue Ĭֵ
     * @return ֵ
     */
    float getFloat(String key, float defaultValue);

    /**
     * ȡòֵָƵĲڣ򷵻<code>0</code>
     * 
     * @param key 
     * @return ֵ
     */
    int getInt(String key);

    /**
     * ȡòֵָƵĲڣ򷵻ָĬֵ
     * 
     * @param key 
     * @param defaultValue Ĭֵ
     * @return ֵ
     */
    int getInt(String key, int defaultValue);

    /**
     * ȡֵָڣ򷵻<code>null</code>
     * 
     * @param key 
     * @return ֵ
     */
    int[] getInts(String key);

    /**
     * ȡֵָڣ򷵻ָĬֵ
     * 
     * @param key 
     * @param defaultValue Ĭֵ
     * @return ֵ
     */
    int[] getInts(String key, int[] defaultValue);

    /**
     * ȡòֵָƵĲڣ򷵻<code>0</code>
     * 
     * @param key 
     * @return ֵ
     */
    long getLong(String key);

    /**
     * ȡòֵָƵĲڣ򷵻ָĬֵ
     * 
     * @param key 
     * @param defaultValue Ĭֵ
     * @return ֵ
     */
    long getLong(String key, long defaultValue);

    /**
     * ȡֵָڣ򷵻<code>null</code>
     * 
     * @param key 
     * @return ֵ
     */
    long[] getLongs(String key);

    /**
     * ȡֵָڣ򷵻ָĬֵ
     * 
     * @param key 
     * @param defaultValue Ĭֵ
     * @return ֵ
     */
    long[] getLongs(String key, long[] defaultValue);

    /**
     * ȡòֵָƵĲڣ򷵻<code>0</code>
     * 
     * @param key 
     * @return ֵ
     */
    short getShort(String key);

    /**
     * ȡòֵָƵĲڣ򷵻ָĬֵ
     * 
     * @param key 
     * @param defaultValue Ĭֵ
     * @return ֵ
     */
    short getShort(String key, short defaultValue);

    /**
     * ȡòֵָƵĲڣ򷵻<code>null</code>
     * 
     * @param key 
     * @return ֵ
     */
    String getString(String key);

    /**
     * ȡòֵָƵĲڣ򷵻ָĬֵ
     * 
     * @param key 
     * @param defaultValue Ĭֵ
     * @return ֵ
     */
    String getString(String key, String defaultValue);

    /**
     * ȡֵָڣ򷵻<code>null</code>
     * 
     * @param key 
     * @return ֵ
     */
    String[] getStrings(String key);

    /**
     * ȡֵָڣ򷵻ָĬֵ
     * 
     * @param key 
     * @param defaultValue Ĭֵ
     * @return ֵ
     */
    String[] getStrings(String key, String[] defaultValue);

    /**
     * ȡòֵָƵĲڣ򷵻<code>null</code> ˷<code>getString</code>
     * һģбʹá
     * 
     * @param key 
     * @return ֵ
     */
    Object get(String key);

    /**
     * ȡֵָڣ򷵻<code>null</code>
     * 
     * @param key 
     * @return ֵ
     */
    Object getObject(String key);

    /**
     * ȡֵָڣ򷵻Ĭֵ
     * 
     * @param key 
     * @param defaultValue Ĭֵ
     * @return ֵ
     */
    Object getObject(String key, Object defaultValue);

    /**
     * ȡֵָڣ򷵻<code>null</code>
     * 
     * @param key 
     * @return ֵ
     */
    Object[] getObjects(String key);

    /**
     * ȡֵָڣ򷵻ָĬֵ
     * 
     * @param key 
     * @param defaultValue Ĭֵ
     * @return ֵ
     */
    Object[] getObjects(String key, Object[] defaultValue);

    /**
     * ȡڡַʹָ<code>DateFormat</code>ڣ򷵻<code>null</code>
     * 
     * @param key 
     * @param format <code>DateFormat</code>
     * @return <code>java.util.Date</code>
     */
    Date getDate(String key, DateFormat format);

    /**
     * ȡڡַʹָ<code>DateFormat</code>ڣ򷵻Ĭֵ
     * 
     * @param key 
     * @param format <code>DateFormat</code>
     * @param defaultValue Ĭֵ
     * @return <code>java.util.Date</code>
     */
    Date getDate(String key, DateFormat format, Date defaultValue);

    /**
     * ȡָ͵Ķ
     */
    <T> T getObjectOfType(String key, Class<T> type);

    /**
     * ȡָ͵Ķ
     */
    <T> T getObjectOfType(String key, Class<T> type, MethodParameter methodParameter, Object[] defaultValues);

    /**
     * ݱ浽object propertiesС
     */
    void setProperties(Object object);

    // =============================================================
    //  Ӻ޸Ĳķ
    // =============================================================

    /**
     * Ӳ/ֵ
     * 
     * @param key 
     * @param value ֵ
     */
    void add(String key, boolean value);

    /**
     * Ӳ/ֵ
     * 
     * @param key 
     * @param value ֵ
     */
    void add(String key, byte value);

    /**
     * Ӳ/ֵ
     * 
     * @param key 
     * @param value ֵ
     */
    void add(String key, char value);

    /**
     * Ӳ/ֵ
     * 
     * @param key 
     * @param value ֵ
     */
    void add(String key, double value);

    /**
     * Ӳ/ֵ
     * 
     * @param key 
     * @param value ֵ
     */
    void add(String key, float value);

    /**
     * Ӳ/ֵ
     * 
     * @param key 
     * @param value ֵ
     */
    void add(String key, int value);

    /**
     * Ӳ/ֵ
     * 
     * @param key 
     * @param value ֵ
     */
    void add(String key, long value);

    /**
     * Ӳ/ֵ
     * 
     * @param key 
     * @param value ֵ
     */
    void add(String key, short value);

    /**
     * Ӳ/ֵ
     * 
     * @param key 
     * @param value ֵ
     */
    void add(String key, Object value);

    /**
     * òֵ<code>add</code>ͬ˷ԭеֵ
     * 
     * @param key 
     * @param value ֵ
     */
    void setString(String key, String value);

    /**
     * òֵ<code>add</code>ͬ˷ԭеֵ
     * 
     * @param key 
     * @param values ֵ
     */
    void setStrings(String key, String[] values);

    /**
     * òֵ<code>add</code>ͬ˷ԭеֵ
     * 
     * @param key 
     * @param value ֵ
     */
    void setObject(String key, Object value);

    /**
     * òֵ<code>add</code>ͬ˷ԭеֵ
     * 
     * @param key 
     * @param value ֵ
     */
    void setObjects(String key, Object[] value);

    // =============================================================
    //  ķ 
    // =============================================================

    /**
     * ɾָƵĲ
     * 
     * @return ԭȺָƶӦĲֵ<code>String[]</code><code>null</code>
     */
    Object remove(String key);

    /**
     * ֵ
     */
    void clear();
}
