/*
 * Copyright 2010 Alibaba Group Holding Limited.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.citrus.service.requestcontext.parser.impl;

import static com.alibaba.citrus.util.Assert.*;

import java.util.AbstractMap;
import java.util.AbstractSet;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.PropertyEditorRegistrar;

import com.alibaba.citrus.service.requestcontext.RequestContext;
import com.alibaba.citrus.service.requestcontext.parser.CookieParser;
import com.alibaba.citrus.service.requestcontext.parser.ParameterParser;
import com.alibaba.citrus.service.requestcontext.parser.ParameterParserFilter;
import com.alibaba.citrus.service.requestcontext.parser.ParserRequestContext;
import com.alibaba.citrus.service.requestcontext.support.AbstractRequestContextWrapper;
import com.alibaba.citrus.service.requestcontext.support.AbstractRequestWrapper;
import com.alibaba.citrus.service.upload.UploadService;
import com.alibaba.citrus.util.StringUtil;

/**
 * Զrequest parameterscookie parameters͸شuploadrequest contextʵ֡
 */
public class ParserRequestContextImpl extends AbstractRequestContextWrapper implements ParserRequestContext {
    private PropertyEditorRegistrar propertyEditorRegistrar;
    private boolean converterQuiet;
    private String caseFolding;
    private boolean autoUpload;
    private boolean unescapeParameters;
    private boolean useServletEngineParser;
    private boolean useBodyEncodingForURI;
    private String uriEncoding;
    private boolean trimming;
    private UploadService upload;
    private ParameterParser parameters;
    private ParameterParserFilter[] filters;
    private String htmlFieldSuffix;
    private CookieParser cookies;

    /**
     * װһ<code>RequestContext</code>
     * 
     * @param wrappedContext װ<code>RequestContext</code>
     */
    public ParserRequestContextImpl(RequestContext wrappedContext) {
        super(wrappedContext);
        setRequest(new RequestWrapper(wrappedContext.getRequest()));
    }

    /**
     * ȡת͵propertyEditorע
     */
    public PropertyEditorRegistrar getPropertyEditorRegistrar() {
        return propertyEditorRegistrar;
    }

    /**
     * ת͵propertyEditorע
     */
    public void setPropertyEditorRegistrar(PropertyEditorRegistrar propertyEditorRegistrar) {
        this.propertyEditorRegistrar = propertyEditorRegistrar;
    }

    /**
     * תʱǷ񲻱ǷĬֵ
     */
    public boolean isConverterQuiet() {
        return converterQuiet;
    }

    /**
     * תʱǷ񲻱ǷĬֵ
     */
    public void setConverterQuiet(boolean converterQuiet) {
        this.converterQuiet = converterQuiet;
    }

    /**
     * ǷԶִUpload
     */
    public boolean isAutoUpload() {
        return autoUpload;
    }

    /**
     * ǷԶִUpload
     */
    public void setAutoUpload(boolean autoUpload) {
        this.autoUpload = autoUpload;
    }

    /**
     * ָķתparameterscookiesƣĬΪСд»ߡ
     */
    public String getCaseFolding() {
        return caseFolding;
    }

    /**
     * ָķתparameterscookiesƣĬΪСд»ߡ
     */
    public void setCaseFolding(String folding) {
        this.caseFolding = folding;
    }

    /**
     * ǷԲHTML entities룬ĬΪ<code>false</code>
     */
    public boolean isUnescapeParameters() {
        return unescapeParameters;
    }

    /**
     * ǷԲHTML entities룬ĬΪ<code>false</code>
     */
    public void setUnescapeParameters(boolean unescapeParameters) {
        this.unescapeParameters = unescapeParameters;
    }

    /**
     * ǷʹservletparserĬΪ<code>false</code>
     */
    public void setUseServletEngineParser(boolean useServletEngineParser) {
        this.useServletEngineParser = useServletEngineParser;
    }

    /**
     * ǷʹservletparserĬΪ<code>false</code>
     */
    public boolean isUseServletEngineParser() {
        return useServletEngineParser;
    }

    /**
     * Ƿrequest.setCharacterEncodingָıqueryĬΪ<code>true</code>
     */
    public boolean isUseBodyEncodingForURI() {
        return useBodyEncodingForURI;
    }

    /**
     * Ƿrequest.setCharacterEncodingָıqueryĬΪ<code>true</code>
     */
    public void setUseBodyEncodingForURI(boolean useBodyEncodingForURI) {
        this.useBodyEncodingForURI = useBodyEncodingForURI;
    }

    /**
     * <code>useServletEngineParser==false</code>
     * <code>useBodyEncodingForURI=false</code>ʱøñGETĲ
     */
    public String getURIEncoding() {
        return uriEncoding;
    }

    /**
     * <code>useServletEngineParser==false</code>
     * <code>useBodyEncodingForURI=false</code>ʱøñGETĲ
     */
    public void setURIEncoding(String uriEncoding) {
        this.uriEncoding = uriEncoding;
    }

    /**
     * ǷtrimmingĬΪ<code>true</code>
     */
    public boolean isTrimming() {
        return trimming;
    }

    /**
     * ǷtrimmingĬΪ<code>true</code>
     */
    public void setTrimming(boolean trimming) {
        this.trimming = trimming;
    }

    /**
     * upload service
     * 
     * @param upload <code>UploadService</code>
     */
    public void setUploadService(UploadService upload) {
        this.upload = upload;
    }

    /**
     * ڹ˲filters
     */
    public void setParameterParserFilters(ParameterParserFilter[] filters) {
        this.filters = filters;
    }

    /**
     * ȡôHTMLֶεĺ׺
     */
    public String getHtmlFieldSuffix() {
        return htmlFieldSuffix;
    }

    /**
     * ôHTMLֶεĺ׺
     */
    public void setHtmlFieldSuffix(String htmlFieldSuffix) {
        this.htmlFieldSuffix = htmlFieldSuffix;
    }

    /**
     * ȡqueryһִд˷ʱrequestȡеĲ
     * 
     * @return <code>ParameterParser</code>ʵ
     */
    public ParameterParser getParameters() {
        if (parameters == null) {
            parameters = new ParameterParserImpl(this, upload, trimming, filters, htmlFieldSuffix);
        }

        return parameters;
    }

    /**
     * ȡcookieһִд˷ʱrequestȡcookies
     * 
     * @return <code>CookieParser</code>ʵ
     */
    public CookieParser getCookies() {
        if (cookies == null) {
            cookies = new CookieParserImpl(this);
        }

        return cookies;
    }

    /**
     * ַָ<code>getCaseFolding()</code>ãתָСдʽ
     * 
     * @param str Ҫתַ
     * @return תַ
     */
    public String convertCase(String str) {
        String folding = getCaseFolding();

        str = StringUtil.trimToEmpty(str);

        if (URL_CASE_FOLDING_LOWER.equals(folding)) {
            str = StringUtil.toLowerCase(str);
        } else if (URL_CASE_FOLDING_LOWER_WITH_UNDERSCORES.equals(folding)) {
            str = StringUtil.toLowerCaseWithUnderscores(str);
        } else if (URL_CASE_FOLDING_UPPER.equals(folding)) {
            str = StringUtil.toUpperCase(str);
        } else if (URL_CASE_FOLDING_UPPER_WITH_UNDERSCORES.equals(folding)) {
            str = StringUtil.toUpperCaseWithUnderscores(str);
        }

        return str;
    }

    /**
     * װrequest
     */
    private class RequestWrapper extends AbstractRequestWrapper {
        private final ParameterMap parameterMap = new ParameterMap();

        public RequestWrapper(HttpServletRequest request) {
            super(ParserRequestContextImpl.this, request);
        }

        @Override
        public String getParameter(String key) {
            return getParameters().getString(key);
        }

        @Override
        public Map<String, String[]> getParameterMap() {
            return parameterMap;
        }

        @Override
        public Enumeration<String> getParameterNames() {
            return new IteratorEnumeration<String>(getParameters().keySet().iterator());
        }

        @Override
        public String[] getParameterValues(String key) {
            return getParameters().getStrings(key);
        }
    }

    private class IteratorEnumeration<E> implements Enumeration<E> {
        private Iterator<E> iterator;

        public IteratorEnumeration(Iterator<E> iterator) {
            this.iterator = iterator;
        }

        public boolean hasMoreElements() {
            return iterator.hasNext();
        }

        public E nextElement() {
            return iterator.next();
        }
    }

    /**
     * һParameterParserΪmap
     */
    private class ParameterMap extends AbstractMap<String, String[]> {
        private final ParameterEntrySet entrySet = new ParameterEntrySet();

        @Override
        public boolean containsKey(Object key) {
            try {
                return getParameters().containsKey((String) key);
            } catch (ClassCastException e) {
                return false;
            }
        }

        @Override
        public String[] get(Object key) {
            try {
                return getParameters().getStrings((String) key);
            } catch (ClassCastException e) {
                return null;
            }
        }

        @Override
        public Set<java.util.Map.Entry<String, String[]>> entrySet() {
            return entrySet;
        }
    }

    private class ParameterEntrySet extends AbstractSet<Map.Entry<String, String[]>> {
        @Override
        public Iterator<Map.Entry<String, String[]>> iterator() {
            final Iterator<String> i = getParameters().keySet().iterator();

            return new Iterator<Map.Entry<String, String[]>>() {
                public boolean hasNext() {
                    return i.hasNext();
                }

                public Entry<String, String[]> next() {
                    return new ParameterEntry(i.next());
                }

                public void remove() {
                    unsupportedOperation();
                }
            };
        }

        @Override
        public int size() {
            return getParameters().size();
        }
    }

    private class ParameterEntry implements Map.Entry<String, String[]> {
        private final String key;

        private ParameterEntry(String key) {
            this.key = key;
        }

        public String getKey() {
            return key;
        }

        public String[] getValue() {
            return getParameters().getStrings(key);
        }

        public String[] setValue(String[] value) {
            unsupportedOperation();
            return null;
        }
    }
}
