/*
 * Copyright 2010 Alibaba Group Holding Limited.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.citrus.service.requestcontext.rundata;

import java.io.IOException;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.alibaba.citrus.service.requestcontext.RequestContext;
import com.alibaba.citrus.service.requestcontext.parser.CookieParser;
import com.alibaba.citrus.service.requestcontext.parser.ParameterParser;
import com.alibaba.citrus.util.io.ByteArray;

/**
 * <code>RunData</code>ṩ˶Գrequest contextһվʽķʷ
 * <p>
 * ⲻһرrequest contextҲֱȡ²request
 * contextﵽͬĹܡrundataΪӦṩһݷʽ
 * </p>
 * 
 * @author Michael Zhou
 */
public interface RunData extends RequestContext {
    // ===================================================
    // HTTP requestϢ
    // ===================================================

    /**
     * ȡquery
     * 
     * @return <code>ParameterParser</code>ʵ
     */
    ParameterParser getParameters();

    /**
     * ȡcookie
     * 
     * @return <code>CookieParser</code>ʵ
     */
    CookieParser getCookies();

    /**
     * ȡõǰڴHTTP
     * 
     * @return HTTP
     */
    HttpServletRequest getRequest();

    /**
     * ȡõǰڴHTTPӦ
     * 
     * @return HTTPӦ
     */
    HttpServletResponse getResponse();

    /**
     * ȡõǰHTTP session
     * 
     * @return HTTP session
     */
    HttpSession getSession();

    /**
     * ȡôǰservletcontainerϢ
     * 
     * @return ǰservletcontainerϢ
     */
    ServletContext getServletContext();

    /**
     * ȡwebӦõ·൱<code>HttpServletRequest.getContextPath</code>صֵ
     * 
     * @return webӦõ·
     */
    String getContextPath();

    /**
     * ȡservlet·൱<code>HttpServletRequest.getServletPath</code>صֵ
     * 
     * @return servlet·
     */
    String getServletPath();

    /**
     * ȡpath info·൱<code>HttpServletRequest.getPathInfo</code>صֵ
     * 
     * @return path info·
     */
    String getPathInfo();

    /**
     * ȡõǰrequest URLquery string
     * 
     * @return ǰrequest URL
     */
    String getRequestURL();

    /**
     * ȡõǰrequest URLquery string
     * 
     * @param withQueryString Ƿquery string
     * @return ǰrequest URL
     */
    String getRequestURL(boolean withQueryString);

    /**
     * ǷΪpost
     */
    boolean isPostRequest();

    // ===================================================
    // Ϣ
    // ===================================================

    /**
     * ȡڷʵǰӦõû
     * 
     * @return û
     */
    User getUser();

    /**
     * ڷʵǰӦõû
     * 
     * @param user û
     */
    void setUser(User user);

    /**
     * ȡú͵ǰ󶨵Ķ󡣵ʱеattributes
     * 
     * @param key key
     * @return keyӦĶ
     */
    Object getAttribute(String key);

    /**
     * ָ󶨵ǰСʱеattributes
     * 
     * @param key key
     * @param object keyӦĶ
     */
    void setAttribute(String key, Object object);

    // ===================================================
    // HTTP responseϢ
    // ===================================================

    /**
     * ȡcontent type
     * 
     * @return content typecharsetĶ
     */
    String getContentType();

    /**
     * content type content typecharset
     * <code>getCharacterEncoding</code>ãcharsetǡ
     * 
     * @param contentType content type
     */
    void setContentType(String contentType);

    /**
     * content type content typecharset
     * <code>getCharacterEncoding</code>ãcharsetǡ
     * <p>
     * <code>appendCharset</code>Ϊ<code>false</code>content
     * typeнcharsetǡ
     * </p>
     * 
     * @param contentType content type
     * @param appendCharset ַ
     */
    void setContentType(String contentType, boolean appendCharset);

    /**
     * ȡresponseַ
     */
    String getCharacterEncoding();

    /**
     * responseַע⣬˷ڵһ<code>getWriter</code>֮ǰִС
     * 
     * @param charset ַcharsetΪ<code>null</code>
     *            contentTypeɾcharset
     */
    void setCharacterEncoding(String charset);

    /**
     * ȡضURI
     * 
     * @return ضURIûض򷵻<code>null</code>
     */
    String getRedirectLocation();

    /**
     * ضURI
     * 
     * @param location ضURI
     * @throws IOException ʧ
     * @throws IllegalStateException responseѾcommitted
     */
    void setRedirectLocation(String location) throws IOException;

    /**
     * оϵͳǷѾض
     * 
     * @return <code>setRedirectLocation</code>ã򷵻<code>true</code>
     */
    boolean isRedirected();

    /**
     * ȡõHTTP status
     * 
     * @return HTTP statusֵ
     */
    int getStatusCode();

    /**
     * HTTP status
     * 
     * @param status HTTP statusֵ
     */
    void setStatusCode(int status);

    // ===================================================
    // Response bufferơ
    // ===================================================

    /**
     * ǷϢڴС
     * 
     * @return ǣ򷵻<code>true</code>
     */
    boolean isBuffering();

    /**
     * bufferģʽó<code>true</code>ʾϢڴУֱԭʼresponseС
     * <p>
     * ˷<code>getOutputStream</code><code>getWriter</code>֮ǰִУ׳
     * <code>IllegalStateException</code>
     * </p>
     * 
     * @param buffering Ƿbuffer
     * @throws IllegalStateException <code>getOutputStream</code>
     *             <code>getWriter</code>Ѿִ
     */
    void setBuffering(boolean buffering);

    /**
     * µbufferϵbuffer
     * 
     * @throws IllegalStateException bufferģʽ<code>getWriter</code>
     *             <code>getOutputStream</code>δ
     */
    void pushBuffer();

    /**
     * bufferջֻһbuffer򵯳ٴһµġ
     * 
     * @return buffer
     * @throws IllegalStateException bufferģʽ<code>getWriter</code>
     *             ã<code>getOutputStream</code>δ
     */
    ByteArray popByteBuffer();

    /**
     * bufferջֻһbuffer򵯳ٴһµġ
     * 
     * @return buffer
     * @throws IllegalStateException bufferģʽ<code>getOutputStream</code>
     *             ã<code>getWriter</code>δ
     */
    String popCharBuffer();

    /**
     * buffersʾϢ
     * 
     * @throws IllegalStateException responseѾcommit
     */
    void resetBuffer();

    /**
     * ַָ<code>getCaseFolding()</code>ãתָСдʽ
     * 
     * @param str Ҫתַ
     * @return תַ
     */
    String convertCase(String str);
}
