/*
 * Copyright 2010 Alibaba Group Holding Limited.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.citrus.service.requestcontext.session;

/**
 * sessionϵͳá
 * 
 * @author Michael Zhou
 */
public interface SessionConfig {
    Integer MAX_INACTIVE_INTERVAL_DEFAULT = 0;
    Long FORCE_EXPIRATION_PERIOD_DEFAULT = 0L;
    String MODEL_KEY_DEFAULT = "SESSION_MODEL";
    Boolean KEEP_IN_TOUCH_DEFAULT = false;

    /**
     * Sessionʱ䣨룩ûʱޣsessionϡֵ<code>0</code>ʾڡ
     */
    int getMaxInactiveInterval();

    /**
     * Sessionǿޣ룩û񣬴session֮ʱ𣬳ޣsessionǿϡֵ
     * <code>0</code>ʾϡ
     */
    long getForceExpirationPeriod();

    /**
     * session modelsessionбļֵSession
     * modelŵǰsession״̬ҲsessionСstore-mappingsУ԰session
     * model䵽һsession storeС
     */
    String getModelKey();

    /**
     * Ƿÿtouch sessionΪ<code>false</code>ֻsessionֵиıʱtouchsession
     * modelcookie storeʱԼ
     */
    boolean isKeepInTouch();

    /**
     * ȡsession IDá
     */
    IdConfig getId();

    /**
     * ȡstores
     */
    StoresConfig getStores();

    /**
     * ȡstore mappings
     */
    StoreMappingsConfig getStoreMappings();

    /**
     * ȡmodel encoders
     */
    SessionModelEncoder[] getSessionModelEncoders();

    /**
     * ȡsessionΪinterceptors
     */
    SessionInterceptor[] getSessionInterceptors();

    /**
     * session IDá
     */
    interface IdConfig {
        Boolean COOKIE_ENABLED_DEFAULT = true;
        Boolean URL_ENCODE_ENABLED_DEFAULT = false;

        /**
         * Ƿsession IDcookieУǣֻܱURLС
         */
        boolean isCookieEnabled();

        /**
         * Ƿְ֧session IDURLС
         */
        boolean isUrlEncodeEnabled();

        /**
         * ȡsession ID cookieá
         */
        CookieConfig getCookie();

        /**
         * ȡsession ID URL encodeá
         */
        UrlEncodeConfig getUrlEncode();

        /**
         * ȡsession ID
         */
        SessionIDGenerator getGenerator();
    }

    /**
     * cookieá
     */
    interface CookieConfig {
        String COOKIE_NAME_DEFAULT = "JSESSIONID";
        String COOKIE_DOMAIN_DEFAULT = null;
        String COOKIE_PATH_DEFAULT = "/";
        Integer COOKIE_MAX_AGE_DEFAULT = 0;
        Boolean COOKIE_HTTP_ONLY_DEFAULT = true;
        Boolean COOKIE_SECURE_DEFAULT = false;

        /**
         * ȡcookieơ
         */
        String getName();

        /**
         * ȡcookieֵ<code>null</code>ʾݵǰԶdomain
         */
        String getDomain();

        /**
         * ȡcookie·
         */
        String getPath();

        /**
         * Cookieʱ䣨룩ֵ<code>0</code>ʾʱcookieĹرնʧ
         */
        int getMaxAge();

        /**
         * cookiehttpOnlyǡIE6°汾УԻXSSΣա
         */
        boolean isHttpOnly();

        /**
         * cookiesecureǡֻhttpsȫܷʸcookie
         */
        boolean isSecure();
    }

    /**
     * url encodeá
     */
    interface UrlEncodeConfig {
        String URL_ENCODE_NAME_DEFAULT = "JSESSIONID";

        /**
         * ȡURL encodeơ
         */
        String getName();
    }

    /**
     * storesá
     */
    interface StoresConfig {
        /**
         * ȡеsession storeơ
         */
        String[] getStoreNames();

        /**
         * ȡָƵĶŵsession store
         */
        SessionStore getStore(String storeName);
    }

    /**
     * store mappingsá
     */
    interface StoreMappingsConfig {
        String MATCHES_ALL_ATTRIBUTES = "*";

        /**
         * ȡָsession attributeƵĶŵsession store
         */
        String getStoreNameForAttribute(String attrName);

        /**
         * ָstoreӦоȷƥattributeơ
         * <p>
         * ڷǾȷƥattributes򷵻<code>null</code>
         * </p>
         */
        String[] getExactMatchedAttributeNames(String storeName);
    }
}
