/*
 * Copyright 2010 Alibaba Group Holding Limited.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.citrus.springext.support;

import org.springframework.beans.factory.BeanNameAware;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;

/**
 * һͨû࣬ʵcontributionࡣԣ
 * <ul>
 * <li>ɸ<code>resolveBeanInterface()</code>ȡýӿڡ</li>
 * <li>ʵĬϵ<code>toString()</code></li>
 * <li>ʵspringĳʼٵڷ</li>
 * <li>ȡspringעbean name</li>
 * </ul>
 * 
 * @author Michael Zhou
 */
public abstract class BeanSupport implements InitializingBean, DisposableBean, BeanNameAware {
    private Class<?> beanInterface;
    private String beanName;
    private boolean initialized;

    /**
     * ȡbeanӿڡ
     */
    public final Class<?> getBeanInterface() {
        if (beanInterface == null) {
            beanInterface = resolveBeanInterface();
        }

        return beanInterface;
    }

    protected Class<?> resolveBeanInterface() {
        return getClass();
    }

    /**
     * ǷѾʼ
     */
    public boolean isInitialized() {
        return initialized;
    }

    /**
     * Ѿʼ׳<code>IllegalStateException</code>쳣
     */
    public void assertInitialized() {
        if (!initialized) {
            throw new IllegalStateException(String.format("Bean instance of %s has not been initialized yet.",
                    getBeanInterface().getName()));
        }
    }

    /**
     * ʼǰִС
     */
    protected void preInit() throws Exception {
    }

    /**
     * ʼbean
     */
    protected void init() throws Exception {
    }

    /**
     * ʼִС
     */
    protected void postInit() throws Exception {
    }

    public final void afterPropertiesSet() throws Exception {
        preInit();
        init();
        initialized = true;
        postInit();
    }

    /**
     * bean
     */
    protected void dispose() {
    }

    /**
     * ǰִС
     */
    protected void preDispose() {
    }

    /**
     * ٺִС
     */
    protected void postDispose() {
    }

    public final void destroy() {
        preDispose();
        dispose();
        initialized = false;
        postDispose();
    }

    /**
     * ȡspringеbeanƣڵԡ
     */
    public String getBeanName() {
        return beanName;
    }

    /**
     * springеbeanƣڵԡ
     */
    public void setBeanName(String beanName) {
        this.beanName = beanName;
    }

    /**
     * תַ
     */
    @Override
    public String toString() {
        return getBeanDescription();
    }

    /**
     * ȡbean<code>beanName + ":" + beanInterfaceName</code>
     */
    protected String getBeanDescription() {
        return getBeanDescription(true);
    }

    /**
     * ȡbean<code>beanName + ":" + beanInterfaceName</code>
     */
    protected String getBeanDescription(boolean simpleName) {
        String interfaceDesc = simpleName ? getBeanInterface().getSimpleName() : getBeanInterface().getName();
        return beanName == null || beanName.contains("(inner bean)") ? interfaceDesc : beanName + ":" + interfaceDesc;
    }
}
