/*
 * Copyright 2010 Alibaba Group Holding Limited.
 * All rights reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *    http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.citrus.springext.support.context;

import static com.alibaba.citrus.util.Assert.*;

import java.lang.reflect.Method;
import java.util.Collection;

import org.springframework.context.ApplicationListener;
import org.springframework.context.support.AbstractApplicationContext;

/**
 * spring3spring2apiȫݣspring2ϱcontext.getApplicationListeners()
 * spring3бNoSuchMethodError÷ķ⡣ȫǨƵspring3Ժ󣬿ɾʵ֡
 * 
 * @author Michael Zhou
 */
class GetApplicationListeners {
    private final AbstractApplicationContext context;
    private final Method getApplicationListenersMethod;

    public GetApplicationListeners(AbstractApplicationContext context) {
        this.context = context;

        Method method = null;

        for (Class<?> clazz = context.getClass(); clazz != null; clazz = clazz.getSuperclass()) {
            try {
                method = clazz.getDeclaredMethod("getApplicationListeners");
                break;
            } catch (Exception e) {
            }
        }

        getApplicationListenersMethod = assertNotNull(method,
                "Could not call method: context.getApplicationListeners()");
    }

    public Collection<ApplicationListener> invoke() {
        try {
            @SuppressWarnings("unchecked")
            Collection<ApplicationListener> listeners = (Collection<ApplicationListener>) getApplicationListenersMethod
                    .invoke(context);

            return listeners;
        } catch (Exception e) {
            throw new RuntimeException("Could not call method: context.getApplicationListeners()", e);
        }
    }
}
