/*
 * Copyright 2010 Alibaba Group Holding Limited.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.citrus.turbine.support;

import static com.alibaba.citrus.util.BasicConstant.*;
import static com.alibaba.citrus.util.CollectionUtil.*;

import java.util.Set;

import com.alibaba.citrus.turbine.Context;
import com.alibaba.citrus.util.ToStringBuilder;
import com.alibaba.citrus.util.ToStringBuilder.MapBuilder;

/**
 * <code>Context</code>ʵ֣ṩ˿Ƕ׵contextơ
 * 
 * @author Michael Zhou
 */
public abstract class AbstractContext implements Context {
    /** contextָkeyڵǰcontextڣڸcontextвҡ */
    private final Context parentContext;

    /**
     * һcontext
     */
    public AbstractContext() {
        this(null);
    }

    /**
     * һcontextָparent context
     */
    public AbstractContext(Context parentContext) {
        this.parentContext = parentContext;
    }

    /**
     * ȡøcontext粻򷵻<code>null</code>
     */
    public Context getParentContext() {
        return parentContext;
    }

    /**
     * һֵ
     */
    public final void put(String key, Object value) {
        if (value == null) {
            remove(key);
        } else {
            internalPut(key, value);
        }
    }

    /**
     * ȡֵָ
     */
    public final Object get(String key) {
        Object value = internalGet(key);

        if (value == null && parentContext != null) {
            return parentContext.get(key);
        }

        return decodeValue(value);
    }

    /**
     * ɾһֵ
     */
    public final void remove(String key) {
        if (parentContext != null && parentContext.containsKey(key)) {
            internalPut(key, NULL_PLACEHOLDER);
        } else {
            internalRemove(key);
        }
    }

    /**
     * жǷָļ
     */
    public final boolean containsKey(String key) {
        boolean containsKey = internalContainsKey(key);

        if (!containsKey && parentContext != null) {
            return parentContext.containsKey(key);
        }

        return containsKey;
    }

    /**
     * ȡkeyļϡ
     */
    public final Set<String> keySet() {
        Set<String> internalKeySet = internalKeySet();
        Set<String> parentKeySet = parentContext == null ? null : parentContext.keySet();

        if (parentKeySet == null || parentKeySet.isEmpty()) {
            return internalKeySet;
        }

        Set<String> newSet = createHashSet();

        newSet.addAll(parentKeySet);
        newSet.addAll(internalKeySet);

        return newSet;
    }

    /**
     * ȡkeyļϡ
     */
    protected abstract Set<String> internalKeySet();

    /**
     * ȡֵָ
     */
    protected abstract Object internalGet(String key);

    /**
     * ɾһֵ
     */
    protected abstract void internalRemove(String key);

    /**
     * жǷָļ
     */
    protected abstract boolean internalContainsKey(String key);

    /**
     * һֵ
     */
    protected abstract void internalPut(String key, Object value);

    /**
     * contextֵΪ<code>NULL_PLACEHOLDER</code>򷵻<code>null</code>
     */
    private Object decodeValue(Object value) {
        return value == NULL_PLACEHOLDER ? null : value;
    }

    @Override
    public String toString() {
        MapBuilder mb;

        if (parentContext == null) {
            mb = getMapBuilder();
        } else {
            mb = new MapBuilder();

            mb.append("parentContext", parentContext);
            mb.append("thisContext", getMapBuilder());
        }

        return new ToStringBuilder().append(getClass().getSimpleName()).append(mb).toString();
    }

    private MapBuilder getMapBuilder() {
        MapBuilder mb = new MapBuilder().setSortKeys(true);

        for (String key : internalKeySet()) {
            mb.append(key, get(key));
        }

        return mb;
    }
}
