/*
 * Copyright 2010 Alibaba Group Holding Limited.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.citrus.util;

import static com.alibaba.citrus.util.CollectionUtil.*;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import com.alibaba.citrus.util.io.StreamUtil;

/**
 * <p>
 * ҲװԴĸࡣ
 * </p>
 * <p>
 * <code>ClassLoaderUtil</code>ԴЧ ൱<code>ClassLoader.loadClass</code>
 * <code>ClassLoader.getResource</code> <code>ClassLoaderUtil</code>
 * ȳԴ<code>Thread.getContextClassLoader()</code>ȡ
 * <code>ClassLoader</code>вװԴ ַڶ༶<code>ClassLoader</code>
 * £ҲԴ
 * </p>
 * <p>
 * :
 * </p>
 * <ul>
 * <li><code>A</code>Ǵϵͳ<code>ClassLoader</code>װ(classpath)</li>
 * <li><code>B</code>Web Applicationеһ࣬servlet<code>ClassLoader</code>
 * ̬װ</li>
 * <li>Դļ<code>C.properties</code>ҲWeb ApplicationУֻservletĶ̬
 * <code>ClassLoader</code>ҵ</li>
 * <li><code>B</code>ù<code>A</code>ķϣͨ<code>A</code>ȡԴļ
 * <code>C.properties</code></li>
 * </ul>
 * <p>
 * <code>A</code>ʹ
 * <code>getClass().getClassLoader().getResource(&quot;C.properties&quot;)</code>
 *  ͻʧܣΪϵͳ<code>ClassLoader</code>ҵԴ
 * AʹClassLoaderUtil.getResource(&quot;C.properties&quot;)ͿҵԴ
 * ΪClassLoaderUtil<code>Thread.currentThead().getContextClassLoader()</code>
 * ȡservlet<code>ClassLoader</code> ӶҵԴļ
 * </p>
 * 
 * @author Michael Zhou
 */
public class ClassLoaderUtil {

    // ==========================================================================
    // ȡcontext class loaderķ 
    // ==========================================================================

    /**
     * ȡõǰ̵߳<code>ClassLoader</code>ҪJDK1.2߰汾JDK֧֡
     * 
     * @return ǰ̵߳<code>ClassLoader</code>
     */
    public static ClassLoader getContextClassLoader() {
        return Thread.currentThread().getContextClassLoader();
    }

    // ==========================================================================
    // װķ 
    // ==========================================================================

    /**
     * ӵǰ̵߳<code>ClassLoader</code>װࡣJDK1.2£൱
     * <code>Class.forName</code>
     * 
     * @param className Ҫװ
     * @return װ
     * @throws ClassNotFoundException ûҵ
     */
    public static Class<?> loadClass(String className) throws ClassNotFoundException {
        return loadClass(className, getContextClassLoader());
    }

    /**
     * ָĵߵ<code>ClassLoader</code>װࡣ
     * 
     * @param className Ҫװ
     * @param referrer ࣬Ϊ<code>null</code>÷൱
     *            <code>Class.forName</code>
     * @return װ
     * @throws ClassNotFoundException ûҵ
     */
    public static Class<?> loadClass(String className, Class<?> referrer) throws ClassNotFoundException {
        ClassLoader classLoader = getReferrerClassLoader(referrer);

        // classLoaderΪnullʾClassLoaderUtilڵclassloaderװأ
        // classloaderδSystem class loader
        return loadClass(className, classLoader);
    }

    /**
     * ָ<code>ClassLoader</code>װࡣδָ<code>ClassLoader</code> װ
     * <code>ClassLoaderUtil</code><code>ClassLoader</code>װ롣
     * 
     * @param className Ҫװ
     * @param classLoader ָ<code>ClassLoader</code>װ࣬Ϊ<code>null</code>
     *            ʾ<code>ClassLoaderUtil</code>ڵclass loaderװ
     * @return װ
     * @throws ClassNotFoundException ûҵ
     */
    public static Class<?> loadClass(String className, ClassLoader classLoader) throws ClassNotFoundException {
        if (className == null) {
            return null;
        }

        if (classLoader == null) {
            return Class.forName(className);
        } else {
            return Class.forName(className, true, classLoader);
        }
    }

    /**
     * ձ׼jar service淶װָƵserviceࡣ
     * <p>
     * class loaderв<code>META-INF/services/serviceId</code>
     * ļΪservice
     * </p>
     * 
     * @param serviceId 
     * @return service class
     * @throws ClassNotFoundException service classҲװserviceļʧ
     */
    public static Class<?> loadServiceClass(String serviceId) throws ClassNotFoundException {
        return loadServiceClass(serviceId, getContextClassLoader());
    }

    /**
     * ձ׼jar service淶װָƵserviceࡣ
     * <p>
     * class loaderв<code>META-INF/services/serviceId</code>
     * ļΪservice
     * </p>
     * 
     * @param serviceId 
     * @param referrer ࣬Ϊ<code>null</code>ʾ<code>ClassLoaderUtil</code>
     *            class loaderҡ
     * @return service class
     * @throws ClassNotFoundException service classҲװserviceļʧ
     */
    public static Class<?> loadServiceClass(String serviceId, Class<?> referrer) throws ClassNotFoundException {
        ClassLoader classLoader = getReferrerClassLoader(referrer);

        // classLoaderΪnullʾClassLoaderUtilڵclassloaderвң
        return loadServiceClass(serviceId, classLoader);
    }

    /**
     * ձ׼jar service淶װָƵserviceࡣ
     * <p>
     * class loaderв<code>META-INF/services/serviceId</code>
     * ļΪservice
     * </p>
     * 
     * @param serviceId 
     * @param classLoader ָclassLoaderвңΪ<code>null</code>ʾ
     *            <code>ClassLoaderUtil</code>class loaderҡ
     * @return service class
     * @throws ClassNotFoundException service classҲװserviceļʧ
     */
    public static Class<?> loadServiceClass(String serviceId, ClassLoader classLoader) throws ClassNotFoundException {
        if (serviceId == null) {
            return null;
        }

        serviceId = "META-INF/services/" + serviceId;

        InputStream istream = getResourceAsStream(serviceId, classLoader);

        if (istream == null) {
            throw new ServiceNotFoundException("Could not find " + serviceId);
        }

        // ļ/META-INF/services/serviceId
        String serviceClassName;

        try {
            serviceClassName = StringUtil.trimToEmpty(StreamUtil.readText(istream, "UTF-8", true));
        } catch (IOException e) {
            throw new ServiceNotFoundException("Failed to load " + serviceId, e);
        }

        return ClassLoaderUtil.loadClass(serviceClassName, classLoader);
    }

    /**
     * ͼװָƵ࣬Ҳٰձ׼jar service淶װָƵserviceࡣ
     * <p>
     * serviceķclass loaderв<code>META-INF/services/serviceId</code>
     * ļΪservice
     * </p>
     * 
     * @param className Ҫװ
     * @param serviceId 
     * @return service class
     * @throws ClassNotFoundException service classҲװserviceļʧ
     */
    public static Class<?> loadServiceClass(String className, String serviceId) throws ClassNotFoundException {
        return loadServiceClass(className, serviceId, getContextClassLoader());
    }

    /**
     * ͼװָƵ࣬Ҳٰձ׼jar service淶װָƵserviceࡣ
     * <p>
     * serviceķclass loaderв<code>META-INF/services/serviceId</code>
     * ļΪservice
     * </p>
     * 
     * @param className Ҫװ
     * @param serviceId 
     * @param referrer ࣬Ϊ<code>null</code>ʾ<code>ClassLoaderUtil</code>
     *            class loaderҡ
     * @return service class
     * @throws ClassNotFoundException service classҲװserviceļʧ
     */
    public static Class<?> loadServiceClass(String className, String serviceId, Class<?> referrer)
            throws ClassNotFoundException {
        ClassLoader classLoader = getReferrerClassLoader(referrer);

        // classLoaderΪnullʾClassLoaderUtilڵclassloaderвң
        return loadServiceClass(className, serviceId, classLoader);
    }

    /**
     * ͼװָƵ࣬Ҳٰձ׼jar service淶װָƵserviceࡣ
     * <p>
     * serviceķclass loaderв<code>META-INF/services/serviceId</code>
     * ļΪservice
     * </p>
     * 
     * @param className Ҫװ
     * @param serviceId 
     * @param classLoader ָclassLoaderвңΪ<code>null</code>ʾ
     *            <code>ClassLoaderUtil</code>class loaderҡ
     * @return service class
     * @throws ClassNotFoundException service classҲװserviceļʧ
     */
    public static Class<?> loadServiceClass(String className, String serviceId, ClassLoader classLoader)
            throws ClassNotFoundException {
        try {
            // ȳװclass
            if (className != null) {
                return loadClass(className, classLoader);
            }
        } catch (ClassNotFoundException e) {
        }

        // ҲԲservice
        return loadServiceClass(serviceId, classLoader);
    }

    /**
     * ȡõߵclass loader
     * 
     * @param referrer 
     * @return ߵclass loaderreferrerΪ<code>null</code>򷵻
     *         <code>null</code>
     */
    private static ClassLoader getReferrerClassLoader(Class<?> referrer) {
        ClassLoader classLoader = null;

        if (referrer != null) {
            classLoader = referrer.getClassLoader();

            // classLoaderΪnull˵referrerbootstrap classloaderװصģ
            // 磺java.lang.String
            if (classLoader == null) {
                classLoader = ClassLoader.getSystemClassLoader();
            }
        }

        return classLoader;
    }

    // ==========================================================================
    // װ벢ʵķ 
    // ==========================================================================

    /**
     * ӵǰ̵߳<code>ClassLoader</code>װಢʵ֮
     * 
     * @param className Ҫʵ
     * @return ָʵ
     * @throws ClassNotFoundException ûҵ
     * @throws ClassInstantiationException ʵʧ
     */
    public static Object newInstance(String className) throws ClassNotFoundException, ClassInstantiationException {
        return newInstance(loadClass(className));
    }

    /**
     * ָĵߵ<code>ClassLoader</code>װಢʵ֮
     * 
     * @param className Ҫʵ
     * @param referrer ࣬Ϊ<code>null</code><code>ClassLoaderUtil</code>
     *            ڵclass loaderװ
     * @return ָʵ
     * @throws ClassNotFoundException ûҵ
     * @throws ClassInstantiationException ʵʧ
     */
    public static Object newInstance(String className, Class<?> referrer) throws ClassNotFoundException,
            ClassInstantiationException {
        return newInstance(loadClass(className, referrer));
    }

    /**
     * ָ<code>ClassLoader</code>װಢʵ֮δָ<code>ClassLoader</code> װ
     * <code>ClassLoaderUtil</code><code>ClassLoader</code>װ롣
     * 
     * @param className Ҫʵ
     * @param classLoader ָ<code>ClassLoader</code>װ࣬Ϊ<code>null</code>
     *            ʾ<code>ClassLoaderUtil</code>ڵclass loaderװ
     * @return ָʵ
     * @throws ClassNotFoundException ûҵ
     * @throws ClassInstantiationException ʵʧ
     */
    public static Object newInstance(String className, ClassLoader classLoader) throws ClassNotFoundException,
            ClassInstantiationException {
        return newInstance(loadClass(className, classLoader));
    }

    /**
     * ָʵ
     * 
     * @param clazz Ҫʵ
     * @return ָʵ
     * @throws ClassInstantiationException ʵʧ
     */
    private static Object newInstance(Class<?> clazz) throws ClassInstantiationException {
        if (clazz == null) {
            return null;
        }

        try {
            return clazz.newInstance();
        } catch (InstantiationException e) {
            throw new ClassInstantiationException("Failed to instantiate class: " + clazz.getName(), e);
        } catch (IllegalAccessException e) {
            throw new ClassInstantiationException("Failed to instantiate class: " + clazz.getName(), e);
        } catch (Exception e) {
            throw new ClassInstantiationException("Failed to instantiate class: " + clazz.getName(), e);
        }
    }

    /**
     * ձ׼jar service淶װָƵserviceಢʵ֮
     * <p>
     * serviceķclass loaderв<code>META-INF/services/serviceId</code>
     * ļΪservice
     * </p>
     * 
     * @param serviceId 
     * @return ʵ
     * @throws ClassNotFoundException service classҲװserviceļʧ
     * @throws ClassInstantiationException ʵʧ
     */
    public static Object newServiceInstance(String serviceId) throws ClassNotFoundException,
            ClassInstantiationException {
        return newInstance(loadServiceClass(serviceId));
    }

    /**
     * ձ׼jar service淶װָƵserviceಢʵ֮
     * <p>
     * serviceķclass loaderв<code>META-INF/services/serviceId</code>
     * ļΪservice
     * </p>
     * 
     * @param serviceId 
     * @param referrer ࣬Ϊ<code>null</code>ʾ<code>ClassLoaderUtil</code>
     *            class loaderҡ
     * @return ʵ
     * @throws ClassNotFoundException service classҲװserviceļʧ
     * @throws ClassInstantiationException ʵʧ
     */
    public static Object newServiceInstance(String serviceId, Class<?> referrer) throws ClassNotFoundException,
            ClassInstantiationException {
        return newInstance(loadServiceClass(serviceId, referrer));
    }

    /**
     * ձ׼jar service淶װָƵserviceಢʵ֮
     * <p>
     * serviceķclass loaderв<code>META-INF/services/serviceId</code>
     * ļΪservice
     * </p>
     * 
     * @param serviceId 
     * @param classLoader ָclassLoaderвңΪ<code>null</code>ʾ
     *            <code>ClassLoaderUtil</code>class loaderҡ
     * @return ʵ
     * @throws ClassNotFoundException service classҲװserviceļʧ
     * @throws ClassInstantiationException ʵʧ
     */
    public static Object newServiceInstance(String serviceId, ClassLoader classLoader) throws ClassNotFoundException,
            ClassInstantiationException {
        return newInstance(loadServiceClass(serviceId, classLoader));
    }

    /**
     * ͼװָƵ࣬Ҳٰձ׼jar service淶װָƵserviceࡣ ҵԺʵ֮
     * <p>
     * serviceķclass loaderв<code>META-INF/services/serviceId</code>
     * ļΪservice
     * </p>
     * 
     * @param className Ҫװ
     * @param serviceId 
     * @return ʵ
     * @throws ClassNotFoundException service classҲװserviceļʧ
     * @throws ClassInstantiationException ʵʧ
     */
    public static Object newServiceInstance(String className, String serviceId) throws ClassNotFoundException,
            ClassInstantiationException {
        return newInstance(loadServiceClass(className, serviceId));
    }

    /**
     * ͼװָƵ࣬Ҳٰձ׼jar service淶װָƵserviceࡣ ҵԺʵ֮
     * <p>
     * serviceķclass loaderв<code>META-INF/services/serviceId</code>
     * ļΪservice
     * </p>
     * 
     * @param className Ҫװ
     * @param serviceId 
     * @param referrer ࣬Ϊ<code>null</code>ʾ<code>ClassLoaderUtil</code>
     *            class loaderҡ
     * @return ʵ
     * @throws ClassNotFoundException service classҲװserviceļʧ
     * @throws ClassInstantiationException ʵʧ
     */
    public static Object newServiceInstance(String className, String serviceId, Class<?> referrer)
            throws ClassNotFoundException, ClassInstantiationException {
        return newInstance(loadServiceClass(className, serviceId, referrer));
    }

    /**
     * ͼװָƵ࣬Ҳٰձ׼jar service淶װָƵserviceࡣ ҵԺʵ֮
     * <p>
     * serviceķclass loaderв<code>META-INF/services/serviceId</code>
     * ļΪservice
     * </p>
     * 
     * @param className Ҫװ
     * @param serviceId 
     * @param classLoader ָclassLoaderвңΪ<code>null</code>ʾ
     *            <code>ClassLoaderUtil</code>class loaderҡ
     * @return ʵ
     * @throws ClassNotFoundException service classҲװserviceļʧ
     * @throws ClassInstantiationException ʵʧ
     */
    public static Object newServiceInstance(String className, String serviceId, ClassLoader classLoader)
            throws ClassNotFoundException, ClassInstantiationException {
        return newInstance(loadServiceClass(className, serviceId, classLoader));
    }

    // ==========================================================================
    // װͲԴļķ 
    // ==========================================================================

    /**
     * <code>ClassLoader</code>ȡresource URL˳:
     * <ol>
     * <li>ڵǰ̵߳<code>ClassLoader</code>вҡ</li>
     * <li>װԼ<code>ClassLoader</code>вҡ</li>
     * <li>ͨ<code>ClassLoader.getSystemResource</code>ҡ</li>
     * </ol>
     * 
     * @param resourceName ҪҵԴ&quot;/&quot;ָıʶַ
     * @return resourceURL飬ûҵ򷵻ؿ顣б֤ظURL
     */
    public static URL[] getResources(String resourceName) {
        List<URL> urls = createLinkedList();
        boolean found = false;

        // Ŵӵǰ̵߳ClassLoaderвҡ
        found = getResources(urls, resourceName, getContextClassLoader(), false);

        // ûҵŴװԼClassLoaderвҡ
        if (!found) {
            found = getResources(urls, resourceName, ClassLoaderUtil.class.getClassLoader(), false);
        }

        // ĳ: ϵͳClassLoaderв(JDK1.2)
        // JDKڲClassLoaderв(JDK1.2)
        if (!found) {
            found = getResources(urls, resourceName, null, true);
        }

        // زظб
        return getDistinctURLs(urls);
    }

    /**
     * ָ<code>ClassLoader</code>ȡresource URL
     * 
     * @param resourceName ҪҵԴ&quot;/&quot;ָıʶַ
     * @param referrer ࣬Ϊ<code>null</code>ʾ<code>ClassLoaderUtil</code>
     *            class loader
     * @return resourceURL飬ûҵ򷵻ؿ顣б֤ظURL
     */
    public static URL[] getResources(String resourceName, Class<?> referrer) {
        ClassLoader classLoader = getReferrerClassLoader(referrer);
        List<URL> urls = createLinkedList();

        getResources(urls, resourceName, classLoader, classLoader == null);

        // زظб
        return getDistinctURLs(urls);
    }

    /**
     * ָ<code>ClassLoader</code>ȡresource URLδָ
     * <code>ClassLoader</code> װ<code>ClassLoaderUtil</code>
     * <code>ClassLoader</code>ȡresource URL
     * 
     * @param resourceName ҪҵԴ&quot;/&quot;ָıʶַ
     * @param classLoader ָ<code>ClassLoader</code>в
     * @return resourceURL飬ûҵ򷵻ؿ顣б֤ظURL
     */
    public static URL[] getResources(String resourceName, ClassLoader classLoader) {
        List<URL> urls = createLinkedList();

        getResources(urls, resourceName, classLoader, classLoader == null);

        // زظб
        return getDistinctURLs(urls);
    }

    /**
     * ָclass loaderвָƵresourceҵresourceURLָļС
     * 
     * @param urlSet resource URLļ
     * @param resourceName Դ
     * @param classLoader װ
     * @param sysClassLoader Ƿsystem class loaderװԴ
     * @return ҵ򷵻<code>true</code>
     */
    private static boolean getResources(List<URL> urlSet, String resourceName, ClassLoader classLoader,
                                        boolean sysClassLoader) {
        if (resourceName == null) {
            return false;
        }

        Enumeration<URL> i = null;

        try {
            if (classLoader != null) {
                i = classLoader.getResources(resourceName);
            } else if (sysClassLoader) {
                i = ClassLoader.getSystemResources(resourceName);
            }
        } catch (IOException e) {
        }

        if (i != null && i.hasMoreElements()) {
            while (i.hasMoreElements()) {
                urlSet.add(i.nextElement());
            }

            return true;
        }

        return false;
    }

    /**
     * ȥURLбеظ
     * 
     * @param urls URLб
     * @return ظURL飬urlsΪ<code>null</code>򷵻ؿ
     */
    private static URL[] getDistinctURLs(List<URL> urls) {
        if (urls == null || urls.size() == 0) {
            return new URL[0];
        }

        Set<URL> urlSet = createHashSet();

        for (Iterator<URL> i = urls.iterator(); i.hasNext();) {
            URL url = i.next();

            if (urlSet.contains(url)) {
                i.remove();
            } else {
                urlSet.add(url);
            }
        }

        return urls.toArray(new URL[urls.size()]);
    }

    /**
     * <p>
     * <code>ClassLoader</code>ȡresource URL˳:
     * </p>
     * <ol>
     * <li>ڵǰ̵߳<code>ClassLoader</code>вҡ</li>
     * <li>װԼ<code>ClassLoader</code>вҡ</li>
     * <li>ͨ<code>ClassLoader.getSystemResource</code>ҡ</li>
     * </ol>
     * 
     * @param resourceName ҪҵԴ&quot;/&quot;ָıʶַ
     * @return resourceURL
     */
    public static URL getResource(String resourceName) {
        if (resourceName == null) {
            return null;
        }

        ClassLoader classLoader = null;
        URL url = null;

        // Ŵӵǰ̵߳ClassLoaderвҡ
        classLoader = getContextClassLoader();

        if (classLoader != null) {
            url = classLoader.getResource(resourceName);

            if (url != null) {
                return url;
            }
        }

        // ûҵŴװԼClassLoaderвҡ
        classLoader = ClassLoaderUtil.class.getClassLoader();

        if (classLoader != null) {
            url = classLoader.getResource(resourceName);

            if (url != null) {
                return url;
            }
        }

        // ĳ: ϵͳClassLoaderв(JDK1.2)
        // JDKڲClassLoaderв(JDK1.2)
        return ClassLoader.getSystemResource(resourceName);
    }

    /**
     * ָ<code>ClassLoader</code>ȡresource URL
     * 
     * @param resourceName ҪҵԴ&quot;/&quot;ָıʶַ
     * @param referrer ࣬Ϊ<code>null</code>ʾ<code>ClassLoaderUtil</code>
     *            class loaderҡ
     * @return resource URLûҵ򷵻<code>null</code>
     */
    public static URL getResource(String resourceName, Class<?> referrer) {
        if (resourceName == null) {
            return null;
        }

        ClassLoader classLoader = getReferrerClassLoader(referrer);

        return classLoader == null ? ClassLoaderUtil.class.getClassLoader().getResource(resourceName) : classLoader
                .getResource(resourceName);
    }

    /**
     * ָ<code>ClassLoader</code>ȡresource URL
     * 
     * @param resourceName ҪҵԴ&quot;/&quot;ָıʶַ
     * @param classLoader ָclassLoaderвңΪ<code>null</code>ʾ
     *            <code>ClassLoaderUtil</code>class loaderҡ
     * @return resource URLûҵ򷵻<code>null</code>
     */
    public static URL getResource(String resourceName, ClassLoader classLoader) {
        if (resourceName == null) {
            return null;
        }

        return classLoader == null ? ClassLoaderUtil.class.getClassLoader().getResource(resourceName) : classLoader
                .getResource(resourceName);
    }

    /**
     * <code>ClassLoader</code>ȡresource ൱
     * <code>getResource(resourceName).openStream()</code>
     * 
     * @param resourceName ҪҵԴ"/"ָıʶַ
     * @return resource
     */
    public static InputStream getResourceAsStream(String resourceName) {
        URL url = getResource(resourceName);

        try {
            if (url != null) {
                return url.openStream();
            }
        } catch (IOException e) {
            // URLʧܡ
        }

        return null;
    }

    /**
     * <code>ClassLoader</code>ȡresource ൱
     * <code>getResource(resourceName,
     * referrer).openStream()</code>
     * 
     * @param resourceName ҪҵԴ"/"ָıʶַ
     * @param referrer ࣬Ϊ<code>null</code>ʾ<code>ClassLoaderUtil</code>
     *            class loaderҡ
     * @return resource
     */
    public static InputStream getResourceAsStream(String resourceName, Class<?> referrer) {
        URL url = getResource(resourceName, referrer);

        try {
            if (url != null) {
                return url.openStream();
            }
        } catch (IOException e) {
            // URLʧܡ
        }

        return null;
    }

    /**
     * <code>ClassLoader</code>ȡresource ൱
     * <code>getResource(resourceName,
     * classLoader).openStream()</code>
     * 
     * @param resourceName ҪҵԴ"/"ָıʶַ
     * @param classLoader ָclassLoaderвңΪ<code>null</code>ʾ
     *            <code>ClassLoaderUtil</code>class loaderҡ
     * @return resource
     */
    public static InputStream getResourceAsStream(String resourceName, ClassLoader classLoader) {
        URL url = getResource(resourceName, classLoader);

        try {
            if (url != null) {
                return url.openStream();
            }
        } catch (IOException e) {
            // URLʧܡ
        }

        return null;
    }

    // ==========================================================================
    // classλá 
    //
    // UNIXwhich 
    // ==========================================================================

    /**
     * ӵǰ̵߳<code>ClassLoader</code>вָƵࡣ
     * 
     * @param className Ҫҵ
     * @return URL飬оϵͳпҵͬ࣬δҵ򷵻һ
     */
    public static URL[] whichClasses(String className) {
        return getResources(ClassUtil.getResourceNameForClass(className));
    }

    /**
     * ӵǰ̵߳<code>ClassLoader</code>вָƵࡣ
     * 
     * @param className Ҫҵ
     * @param referrer ࣬Ϊ<code>null</code>ʾ<code>ClassLoaderUtil</code>
     *            class loaderҡ
     * @return URL飬оϵͳпҵͬ࣬δҵ򷵻һ
     */
    public static URL[] whichClasses(String className, Class<?> referrer) {
        return getResources(ClassUtil.getResourceNameForClass(className), referrer);
    }

    /**
     * ӵǰ̵߳<code>ClassLoader</code>вָƵࡣ
     * 
     * @param className Ҫҵ
     * @param classLoader ָclassLoaderвңΪ<code>null</code>ʾ
     *            <code>ClassLoaderUtil</code>class loaderҡ
     * @return URL飬оϵͳпҵͬ࣬δҵ򷵻һ
     */
    public static URL[] whichClasses(String className, ClassLoader classLoader) {
        return getResources(ClassUtil.getResourceNameForClass(className), classLoader);
    }

    /**
     * ӵǰ̵߳<code>ClassLoader</code>вָƵࡣ
     * 
     * @param className Ҫҵ
     * @return ļURLδҵ򷵻<code>null</code>
     */
    public static URL whichClass(String className) {
        return getResource(ClassUtil.getResourceNameForClass(className));
    }

    /**
     * ӵǰ̵߳<code>ClassLoader</code>вָƵࡣ
     * 
     * @param className Ҫҵ
     * @param referrer ࣬Ϊ<code>null</code>ʾ<code>ClassLoaderUtil</code>
     *            class loaderҡ
     * @return ļURLδҵ򷵻<code>null</code>
     */
    public static URL whichClass(String className, Class<?> referrer) {
        return getResource(ClassUtil.getResourceNameForClass(className), referrer);
    }

    /**
     * ӵǰ̵߳<code>ClassLoader</code>вָƵࡣ
     * 
     * @param className Ҫҵ
     * @param classLoader ָclassLoaderвңΪ<code>null</code>ʾ
     *            <code>ClassLoaderUtil</code>class loaderҡ
     * @return ļURLδҵ򷵻<code>null</code>
     */
    public static URL whichClass(String className, ClassLoader classLoader) {
        return getResource(ClassUtil.getResourceNameForClass(className), classLoader);
    }
}
