/*
 * Copyright 2010 Alibaba Group Holding Limited.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.citrus.util;

import static com.alibaba.citrus.util.CollectionUtil.*;
import static com.alibaba.citrus.util.ObjectUtil.*;

import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Collections;
import java.util.List;
import java.util.Set;

import com.alibaba.citrus.util.i18n.LocaleUtil;

/**
 * ȡйϵͳϢĹࡣ
 * <p>
 * ΪJavaȫԭȡ<code>System.getProperty</code>Ӧֶνó
 * <code>null</code> Ϣ<code>System.err</code>С
 * </p>
 * 
 * @author Michael Zhou
 */
public class SystemUtil {
    private static final JvmSpecInfo JVM_SPEC_INFO = new JvmSpecInfo();
    private static final JvmInfo JVM_INFO = new JvmInfo();
    private static final JavaSpecInfo JAVA_SPEC_INFO = new JavaSpecInfo();
    private static final JavaInfo JAVA_INFO = new JavaInfo();
    private static final OsInfo OS_INFO = new OsInfo();
    private static final UserInfo USER_INFO = new UserInfo();
    private static final HostInfo HOST_INFO = new HostInfo();
    private static final JavaRuntimeInfo JAVA_RUNTIME_INFO = new JavaRuntimeInfo();

    /**
     * ȡJava Virtual Machine SpecificationϢ
     * 
     * @return <code>JvmSpecInfo</code>
     */
    public static final JvmSpecInfo getJvmSpecInfo() {
        return JVM_SPEC_INFO;
    }

    /**
     * ȡJava Virtual Machine ImplementationϢ
     * 
     * @return <code>JvmInfo</code>
     */
    public static final JvmInfo getJvmInfo() {
        return JVM_INFO;
    }

    /**
     * ȡJava SpecificationϢ
     * 
     * @return <code>JavaSpecInfo</code>
     */
    public static final JavaSpecInfo getJavaSpecInfo() {
        return JAVA_SPEC_INFO;
    }

    /**
     * ȡJava ImplementationϢ
     * 
     * @return <code>JavaInfo</code>
     */
    public static final JavaInfo getJavaInfo() {
        return JAVA_INFO;
    }

    /**
     * ȡõǰеJREϢ
     * 
     * @return <code>JreInfo</code>
     */
    public static final JavaRuntimeInfo getJavaRuntimeInfo() {
        return JAVA_RUNTIME_INFO;
    }

    /**
     * ȡOSϢ
     * 
     * @return <code>OsInfo</code>
     */
    public static final OsInfo getOsInfo() {
        return OS_INFO;
    }

    /**
     * ȡUserϢ
     * 
     * @return <code>UserInfo</code>
     */
    public static final UserInfo getUserInfo() {
        return USER_INFO;
    }

    /**
     * ȡHostϢ
     * 
     * @return <code>HostInfo</code>
     */
    public static final HostInfo getHostInfo() {
        return HOST_INFO;
    }

    /**
     * Java Virutal Machine SpecificationϢ
     */
    public static final class JvmSpecInfo {
        private final String JAVA_VM_SPECIFICATION_NAME = getSystemProperty("java.vm.specification.name", false);
        private final String JAVA_VM_SPECIFICATION_VERSION = getSystemProperty("java.vm.specification.version", false);
        private final String JAVA_VM_SPECIFICATION_VENDOR = getSystemProperty("java.vm.specification.vendor", false);

        /**
         * ֹ紴˶
         */
        private JvmSpecInfo() {
        }

        /**
         * ȡõǰJVM spec.ƣȡϵͳԣ<code>java.vm.specification.name</code>
         * <p>
         * Sun JDK 1.4.2<code>"Java Virtual Machine Specification"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getName() {
            return JAVA_VM_SPECIFICATION_NAME;
        }

        /**
         * ȡõǰJVM spec.İ汾ȡϵͳԣ<code>java.vm.specification.version</code>
         * <p>
         * Sun JDK 1.4.2<code>"1.0"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getVersion() {
            return JAVA_VM_SPECIFICATION_VERSION;
        }

        /**
         * ȡõǰJVM spec.ĳ̣ȡϵͳԣ<code>java.vm.specification.vendor</code>
         * <p>
         * Sun JDK 1.4.2<code>"Sun Microsystems Inc."</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getVendor() {
            return JAVA_VM_SPECIFICATION_VENDOR;
        }

        /**
         * Java Virutal Machine SpecificationϢתַ
         * 
         * @return JVM spec.ַʾ
         */
        @Override
        public final String toString() {
            StringBuilder buffer = new StringBuilder();

            append(buffer, "JavaVM Spec. Name:    ", getName());
            append(buffer, "JavaVM Spec. Version: ", getVersion());
            append(buffer, "JavaVM Spec. Vendor:  ", getVendor());

            return buffer.toString();
        }
    }

    /**
     * Java Virtual Machine ImplementationϢ
     */
    public static final class JvmInfo {
        private final String JAVA_VM_NAME = getSystemProperty("java.vm.name", false);
        private final String JAVA_VM_VERSION = getSystemProperty("java.vm.version", false);
        private final String JAVA_VM_VENDOR = getSystemProperty("java.vm.vendor", false);
        private final String JAVA_VM_INFO = getSystemProperty("java.vm.info", false);

        /**
         * ֹ紴˶
         */
        private JvmInfo() {
        }

        /**
         * ȡõǰJVM impl.ƣȡϵͳԣ<code>java.vm.name</code>
         * <p>
         * Sun JDK 1.4.2<code>"Java HotSpot(TM) Client VM"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getName() {
            return JAVA_VM_NAME;
        }

        /**
         * ȡõǰJVM impl.İ汾ȡϵͳԣ<code>java.vm.version</code>
         * <p>
         * Sun JDK 1.4.2<code>"1.4.2-b28"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getVersion() {
            return JAVA_VM_VERSION;
        }

        /**
         * ȡõǰJVM impl.ĳ̣ȡϵͳԣ<code>java.vm.vendor</code>
         * <p>
         * Sun JDK 1.4.2<code>"Sun Microsystems Inc."</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getVendor() {
            return JAVA_VM_VENDOR;
        }

        /**
         * ȡõǰJVM impl.Ϣȡϵͳԣ<code>java.vm.info</code>
         * <p>
         * Sun JDK 1.4.2<code>"mixed mode"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getInfo() {
            return JAVA_VM_INFO;
        }

        /**
         * Java Virutal Machine ImplementationϢתַ
         * 
         * @return JVM impl.ַʾ
         */
        @Override
        public final String toString() {
            StringBuilder buffer = new StringBuilder();

            append(buffer, "JavaVM Name:    ", getName());
            append(buffer, "JavaVM Version: ", getVersion());
            append(buffer, "JavaVM Vendor:  ", getVendor());
            append(buffer, "JavaVM Info:    ", getInfo());

            return buffer.toString();
        }
    }

    /**
     * Java SpecificationϢ
     */
    public static final class JavaSpecInfo {
        private final String JAVA_SPECIFICATION_NAME = getSystemProperty("java.specification.name", false);
        private final String JAVA_SPECIFICATION_VERSION = getSystemProperty("java.specification.version", false);
        private final String JAVA_SPECIFICATION_VENDOR = getSystemProperty("java.specification.vendor", false);

        /**
         * ֹ紴˶
         */
        private JavaSpecInfo() {
        }

        /**
         * ȡõǰJava Spec.ƣȡϵͳԣ<code>java.specification.name</code>
         * <p>
         * Sun JDK 1.4.2<code>"Java Platform API Specification"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getName() {
            return JAVA_SPECIFICATION_NAME;
        }

        /**
         * ȡõǰJava Spec.İ汾ȡϵͳԣ<code>java.specification.version</code>
         * <p>
         * Sun JDK 1.4.2<code>"1.4"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.3
         */
        public final String getVersion() {
            return JAVA_SPECIFICATION_VERSION;
        }

        /**
         * ȡõǰJava Spec.ĳ̣ȡϵͳԣ<code>java.specification.vendor</code>
         * <p>
         * Sun JDK 1.4.2<code>"Sun Microsystems Inc."</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getVendor() {
            return JAVA_SPECIFICATION_VENDOR;
        }

        /**
         * Java SpecificationϢתַ
         * 
         * @return JVM spec.ַʾ
         */
        @Override
        public final String toString() {
            StringBuilder buffer = new StringBuilder();

            append(buffer, "Java Spec. Name:    ", getName());
            append(buffer, "Java Spec. Version: ", getVersion());
            append(buffer, "Java Spec. Vendor:  ", getVendor());

            return buffer.toString();
        }
    }

    /**
     * Java ImplementationϢ
     */
    public static final class JavaInfo {
        private final String JAVA_VERSION = getSystemProperty("java.version", false);
        private final float JAVA_VERSION_FLOAT = getJavaVersionAsFloat();
        private final int JAVA_VERSION_INT = getJavaVersionAsInt();
        private final String JAVA_VENDOR = getSystemProperty("java.vendor", false);
        private final String JAVA_VENDOR_URL = getSystemProperty("java.vendor.url", false);
        private final boolean IS_JAVA_1_1 = getJavaVersionMatches("1.1");
        private final boolean IS_JAVA_1_2 = getJavaVersionMatches("1.2");
        private final boolean IS_JAVA_1_3 = getJavaVersionMatches("1.3");
        private final boolean IS_JAVA_1_4 = getJavaVersionMatches("1.4");
        private final boolean IS_JAVA_1_5 = getJavaVersionMatches("1.5");

        /**
         * ֹ紴˶
         */
        private JavaInfo() {
        }

        /**
         * ȡõǰJava impl.İ汾ȡϵͳԣ<code>java.version</code>
         * <p>
         * Sun JDK 1.4.2<code>"1.4.2"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getVersion() {
            return JAVA_VERSION;
        }

        /**
         * ȡõǰJava impl.İ汾ȡϵͳԣ<code>java.version</code>
         * <p>
         * 磺
         * <ul>
         * <li>JDK 1.2<code>1.2f</code></li>
         * <li>JDK 1.3.1<code>1.31f</code></li>
         * </ul>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>0</code>
         * @since Java 1.1
         */
        public final float getVersionFloat() {
            return JAVA_VERSION_FLOAT;
        }

        /**
         * ȡõǰJava impl.İ汾ȡϵͳԣ<code>java.version</code>
         * <p>
         * 磺
         * <ul>
         * <li>JDK 1.2<code>120</code></li>
         * <li>JDK 1.3.1<code>131</code></li>
         * </ul>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>0</code>
         * @since Java 1.1
         */
        public final int getVersionInt() {
            return JAVA_VERSION_INT;
        }

        /**
         * ȡõǰJava impl.İ汾<code>float</code>ֵ
         * 
         * @return Java汾<code>float</code>ֵ<code>0</code>
         */
        private final float getJavaVersionAsFloat() {
            if (JAVA_VERSION == null) {
                return 0f;
            }

            String str = JAVA_VERSION.substring(0, 3);

            if (JAVA_VERSION.length() >= 5) {
                str = str + JAVA_VERSION.substring(4, 5);
            }

            return Float.parseFloat(str);
        }

        /**
         * ȡõǰJava impl.İ汾<code>int</code>ֵ
         * 
         * @return Java汾<code>int</code>ֵ<code>0</code>
         */
        private final int getJavaVersionAsInt() {
            if (JAVA_VERSION == null) {
                return 0;
            }

            String str = JAVA_VERSION.substring(0, 1);

            str = str + JAVA_VERSION.substring(2, 3);

            if (JAVA_VERSION.length() >= 5) {
                str = str + JAVA_VERSION.substring(4, 5);
            } else {
                str = str + "0";
            }

            return Integer.parseInt(str);
        }

        /**
         * ȡõǰJava impl.ĳ̣ȡϵͳԣ<code>java.vendor</code>
         * <p>
         * Sun JDK 1.4.2<code>"Sun Microsystems Inc."</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getVendor() {
            return JAVA_VENDOR;
        }

        /**
         * ȡõǰJava impl.ĳվURLȡϵͳԣ<code>java.vendor.url</code>
         * <p>
         * Sun JDK 1.4.2<code>"http://java.sun.com/"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getVendorURL() {
            return JAVA_VENDOR_URL;
        }

        /**
         * жϵǰJavaİ汾
         * <p>
         * ȡϵͳ<code>java.version</code>ΪJavaȫƣǷ
         * <code>false</code>
         * </p>
         * 
         * @return ǰJava汾Ϊ1.1򷵻<code>true</code>
         */
        public final boolean isJava11() {
            return IS_JAVA_1_1;
        }

        /**
         * жϵǰJavaİ汾
         * <p>
         * ȡϵͳ<code>java.version</code>ΪJavaȫƣǷ
         * <code>false</code>
         * </p>
         * 
         * @return ǰJava汾Ϊ1.2򷵻<code>true</code>
         */
        public final boolean isJava12() {
            return IS_JAVA_1_2;
        }

        /**
         * жϵǰJavaİ汾
         * <p>
         * ȡϵͳ<code>java.version</code>ΪJavaȫƣǷ
         * <code>false</code>
         * </p>
         * 
         * @return ǰJava汾Ϊ1.3򷵻<code>true</code>
         */
        public final boolean isJava13() {
            return IS_JAVA_1_3;
        }

        /**
         * жϵǰJavaİ汾
         * <p>
         * ȡϵͳ<code>java.version</code>ΪJavaȫƣǷ
         * <code>false</code>
         * </p>
         * 
         * @return ǰJava汾Ϊ1.4򷵻<code>true</code>
         */
        public final boolean isJava14() {
            return IS_JAVA_1_4;
        }

        /**
         * жϵǰJavaİ汾
         * <p>
         * ȡϵͳ<code>java.version</code>ΪJavaȫƣǷ
         * <code>false</code>
         * </p>
         * 
         * @return ǰJava汾Ϊ1.5򷵻<code>true</code>
         */
        public final boolean isJava15() {
            return IS_JAVA_1_5;
        }

        /**
         * ƥ䵱ǰJavaİ汾
         * 
         * @param versionPrefix Java汾ǰ׺
         * @return 汾ƥ䣬򷵻<code>true</code>
         */
        private final boolean getJavaVersionMatches(String versionPrefix) {
            if (JAVA_VERSION == null) {
                return false;
            }

            return JAVA_VERSION.startsWith(versionPrefix);
        }

        /**
         * жǰJavaİ汾Ƿڵָİ汾š
         * <p>
         * 磺
         * </p>
         * <ul>
         * <li>JDK 1.2<code>isJavaVersionAtLeast(1.2f)</code></li>
         * <li>JDK 1.2.1<code>isJavaVersionAtLeast(1.31f)</code></li>
         * </ul>
         * 
         * @param requiredVersion Ҫİ汾
         * @return ǰJava汾ڻָİ汾򷵻<code>true</code>
         */
        public final boolean isJavaVersionAtLeast(float requiredVersion) {
            return getVersionFloat() >= requiredVersion;
        }

        /**
         * жǰJavaİ汾Ƿڵָİ汾š
         * <p>
         * 磺
         * </p>
         * <ul>
         * <li>JDK 1.2<code>isJavaVersionAtLeast(120)</code></li>
         * <li>JDK 1.2.1<code>isJavaVersionAtLeast(131)</code></li>
         * </ul>
         * 
         * @param requiredVersion Ҫİ汾
         * @return ǰJava汾ڻָİ汾򷵻<code>true</code>
         */
        public final boolean isJavaVersionAtLeast(int requiredVersion) {
            return getVersionInt() >= requiredVersion;
        }

        /**
         * Java ImplementationϢתַ
         * 
         * @return JVM impl.ַʾ
         */
        @Override
        public final String toString() {
            StringBuilder buffer = new StringBuilder();

            append(buffer, "Java Version:    ", getVersion());
            append(buffer, "Java Vendor:     ", getVendor());
            append(buffer, "Java Vendor URL: ", getVendorURL());

            return buffer.toString();
        }
    }

    /**
     * ǰеJREϢ
     */
    public static final class JavaRuntimeInfo {
        private final String JAVA_RUNTIME_NAME = getSystemProperty("java.runtime.name", false);
        private final String JAVA_RUNTIME_VERSION = getSystemProperty("java.runtime.version", false);
        private final String JAVA_HOME = getSystemProperty("java.home", false);
        private final String JAVA_EXT_DIRS = getSystemProperty("java.ext.dirs", false);
        private final String JAVA_ENDORSED_DIRS = getSystemProperty("java.endorsed.dirs", false);
        private final String JAVA_CLASS_PATH = getSystemProperty("java.class.path", false);
        private final String JAVA_CLASS_VERSION = getSystemProperty("java.class.version", false);
        private final String JAVA_LIBRARY_PATH = getSystemProperty("java.library.path", false);

        /**
         * ֹ紴˶
         */
        private JavaRuntimeInfo() {
        }

        /**
         * ȡõǰJREƣȡϵͳԣ<code>java.runtime.name</code>
         * <p>
         * Sun JDK 1.4.2
         * <code>"Java(TM) 2 Runtime Environment, Standard Edition"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.3
         */
        public final String getName() {
            return JAVA_RUNTIME_NAME;
        }

        /**
         * ȡõǰJREİ汾ȡϵͳԣ<code>java.runtime.version</code>
         * <p>
         * Sun JDK 1.4.2<code>"1.4.2-b28"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.3
         */
        public final String getVersion() {
            return JAVA_RUNTIME_VERSION;
        }

        /**
         * ȡõǰJREİװĿ¼ȡϵͳԣ<code>java.home</code>
         * <p>
         * Sun JDK 1.4.2<code>"/opt/jdk1.4.2/jre"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getHomeDir() {
            return JAVA_HOME;
        }

        /**
         * ȡõǰJREչĿ¼бȡϵͳԣ<code>java.ext.dirs</code>
         * <p>
         * Sun JDK 1.4.2<code>"/opt/jdk1.4.2/jre/lib/ext:..."</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.3
         */
        public final String getExtDirs() {
            return JAVA_EXT_DIRS;
        }

        /**
         * ȡõǰJREendorsedĿ¼бȡϵͳԣ<code>java.endorsed.dirs</code>
         * <p>
         * Sun JDK 1.4.2<code>"/opt/jdk1.4.2/jre/lib/endorsed:..."</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.4
         */
        public final String getEndorsedDirs() {
            return JAVA_ENDORSED_DIRS;
        }

        /**
         * ȡõǰJREϵͳclasspathȡϵͳԣ<code>java.class.path</code>
         * <p>
         * 磺<code>"/home/admin/myclasses:/home/admin/..."</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getClassPath() {
            return JAVA_CLASS_PATH;
        }

        /**
         * ȡõǰJREϵͳclasspathȡϵͳԣ<code>java.class.path</code>
         * <p>
         * 磺<code>"/home/admin/myclasses:/home/admin/..."</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String[] getClassPathArray() {
            return StringUtil.split(getClassPath(), getOsInfo().getPathSeparator());
        }

        /**
         * ȡõǰJREclassļʽİ汾ȡϵͳԣ<code>java.class.version</code>
         * <p>
         * Sun JDK 1.4.2<code>"48.0"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getClassVersion() {
            return JAVA_CLASS_VERSION;
        }

        /**
         * ȡõǰJRElibrary·ȡϵͳԣ<code>java.library.path</code>
         * <p>
         * Sun JDK 1.4.2<code>"/opt/jdk1.4.2/bin:..."</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getLibraryPath() {
            return JAVA_LIBRARY_PATH;
        }

        /**
         * ȡõǰJRElibrary·ȡϵͳԣ<code>java.library.path</code>
         * <p>
         * Sun JDK 1.4.2<code>"/opt/jdk1.4.2/bin:..."</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String[] getLibraryPathArray() {
            return StringUtil.split(getLibraryPath(), getOsInfo().getPathSeparator());
        }

        /**
         * ȡõǰJREURLЭpackagesбȡϵͳԣ<code>java.library.path</code>
         * <p>
         * Sun JDK 1.4.2<code>"sun.net.www.protocol|..."</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getProtocolPackages() {
            return getSystemProperty("java.protocol.handler.pkgs", true);
        }

        /**
         * ǰеJREϢתַ
         * 
         * @return JREϢַʾ
         */
        @Override
        public final String toString() {
            StringBuilder buffer = new StringBuilder();

            append(buffer, "Java Runtime Name:      ", getName());
            append(buffer, "Java Runtime Version:   ", getVersion());
            append(buffer, "Java Home Dir:          ", getHomeDir());
            append(buffer, "Java Extension Dirs:    ", getExtDirs());
            append(buffer, "Java Endorsed Dirs:     ", getEndorsedDirs());
            append(buffer, "Java Class Path:        ", getClassPath());
            append(buffer, "Java Class Version:     ", getClassVersion());
            append(buffer, "Java Library Path:      ", getLibraryPath());
            append(buffer, "Java Protocol Packages: ", getProtocolPackages());

            return buffer.toString();
        }
    }

    /**
     * ǰOSϢ
     */
    public static final class OsInfo {
        private final String OS_VERSION = getSystemProperty("os.version", false);
        private final String OS_ARCH = getSystemProperty("os.arch", false);
        private final String OS_NAME = getSystemProperty("os.name", false);
        private final boolean IS_OS_AIX = getOSMatches("AIX");
        private final boolean IS_OS_HP_UX = getOSMatches("HP-UX");
        private final boolean IS_OS_IRIX = getOSMatches("Irix");
        private final boolean IS_OS_LINUX = getOSMatches("Linux") || getOSMatches("LINUX");
        private final boolean IS_OS_MAC = getOSMatches("Mac");
        private final boolean IS_OS_MAC_OSX = getOSMatches("Mac OS X");
        private final boolean IS_OS_OS2 = getOSMatches("OS/2");
        private final boolean IS_OS_SOLARIS = getOSMatches("Solaris");
        private final boolean IS_OS_SUN_OS = getOSMatches("SunOS");
        private final boolean IS_OS_WINDOWS = getOSMatches("Windows");
        private final boolean IS_OS_WINDOWS_2000 = getOSMatches("Windows", "5.0");
        private final boolean IS_OS_WINDOWS_95 = getOSMatches("Windows 9", "4.0");
        private final boolean IS_OS_WINDOWS_98 = getOSMatches("Windows 9", "4.1");
        private final boolean IS_OS_WINDOWS_ME = getOSMatches("Windows", "4.9");
        private final boolean IS_OS_WINDOWS_NT = getOSMatches("Windows NT");
        private final boolean IS_OS_WINDOWS_XP = getOSMatches("Windows", "5.1");

        // ڸıfile.encodingԲıϵͳַ룬Ϊ˱һ£ͨLocaleUtilȡϵͳĬϱ롣
        private final String FILE_ENCODING = LocaleUtil.getSystem().getCharset().name();
        private final String FILE_SEPARATOR = getSystemProperty("file.separator", false);
        private final String LINE_SEPARATOR = getSystemProperty("line.separator", false);
        private final String PATH_SEPARATOR = getSystemProperty("path.separator", false);

        /**
         * ֹ紴˶
         */
        private OsInfo() {
        }

        /**
         * ȡõǰOSļܹȡϵͳԣ<code>os.arch</code>
         * <p>
         * 磺<code>"x86"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getArch() {
            return OS_ARCH;
        }

        /**
         * ȡõǰOSƣȡϵͳԣ<code>os.name</code>
         * <p>
         * 磺<code>"Windows XP"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getName() {
            return OS_NAME;
        }

        /**
         * ȡõǰOSİ汾ȡϵͳԣ<code>os.version</code>
         * <p>
         * 磺<code>"5.1"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getVersion() {
            return OS_VERSION;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪAIX򷵻<code>true</code>
         */
        public final boolean isAix() {
            return IS_OS_AIX;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪHP-UX򷵻<code>true</code>
         */
        public final boolean isHpUx() {
            return IS_OS_HP_UX;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪIRIX򷵻<code>true</code>
         */
        public final boolean isIrix() {
            return IS_OS_IRIX;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪLinux򷵻<code>true</code>
         */
        public final boolean isLinux() {
            return IS_OS_LINUX;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪMac򷵻<code>true</code>
         */
        public final boolean isMac() {
            return IS_OS_MAC;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪMacOS X򷵻<code>true</code>
         */
        public final boolean isMacOsX() {
            return IS_OS_MAC_OSX;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪOS2򷵻<code>true</code>
         */
        public final boolean isOs2() {
            return IS_OS_OS2;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪSolaris򷵻<code>true</code>
         */
        public final boolean isSolaris() {
            return IS_OS_SOLARIS;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪSun OS򷵻<code>true</code>
         */
        public final boolean isSunOS() {
            return IS_OS_SUN_OS;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪWindows򷵻<code>true</code>
         */
        public final boolean isWindows() {
            return IS_OS_WINDOWS;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪWindows 2000򷵻<code>true</code>
         */
        public final boolean isWindows2000() {
            return IS_OS_WINDOWS_2000;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪWindows 95򷵻<code>true</code>
         */
        public final boolean isWindows95() {
            return IS_OS_WINDOWS_95;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪWindows 98򷵻<code>true</code>
         */
        public final boolean isWindows98() {
            return IS_OS_WINDOWS_98;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪWindows ME򷵻<code>true</code>
         */
        public final boolean isWindowsME() {
            return IS_OS_WINDOWS_ME;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪWindows NT򷵻<code>true</code>
         */
        public final boolean isWindowsNT() {
            return IS_OS_WINDOWS_NT;
        }

        /**
         * жϵǰOS͡
         * <p>
         * ȡϵͳ<code>os.name</code>ΪJavaȫƣǷ<code>false</code>
         * </p>
         * 
         * @return ǰOSΪWindows XP򷵻<code>true</code>
         */
        public final boolean isWindowsXP() {
            return IS_OS_WINDOWS_XP;
        }

        /**
         * ƥOSơ
         * 
         * @param osNamePrefix OSǰ׺
         * @return ƥ䣬򷵻<code>true</code>
         */
        private final boolean getOSMatches(String osNamePrefix) {
            if (OS_NAME == null) {
                return false;
            }

            return OS_NAME.startsWith(osNamePrefix);
        }

        /**
         * ƥOSơ
         * 
         * @param osNamePrefix OSǰ׺
         * @param osVersionPrefix OS汾ǰ׺
         * @return ƥ䣬򷵻<code>true</code>
         */
        private final boolean getOSMatches(String osNamePrefix, String osVersionPrefix) {
            if (OS_NAME == null || OS_VERSION == null) {
                return false;
            }

            return OS_NAME.startsWith(osNamePrefix) && OS_VERSION.startsWith(osVersionPrefix);
        }

        /**
         * ȡOSĬַ뼯ȡϵͳԣ<code>file.encoding</code>
         * <p>
         * ַΪǰJVMתֽ/ַĬϷʽ 磺<code>GBK</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getFileEncoding() {
            return FILE_ENCODING;
        }

        /**
         * ȡOSļ·ķָȡϵͳԣ<code>file.separator</code>
         * <p>
         * 磺UnixΪ<code>"/"</code>WindowsΪ<code>"\\"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getFileSeparator() {
            return FILE_SEPARATOR;
        }

        /**
         * ȡOSıļзȡϵͳԣ<code>line.separator</code>
         * <p>
         * 磺UnixΪ<code>"\n"</code>WindowsΪ<code>"\r\n"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getLineSeparator() {
            return LINE_SEPARATOR;
        }

        /**
         * ȡOS·ָȡϵͳԣ<code>path.separator</code>
         * <p>
         * 磺UnixΪ<code>":"</code>WindowsΪ<code>";"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getPathSeparator() {
            return PATH_SEPARATOR;
        }

        /**
         * OSϢתַ
         * 
         * @return OSַʾ
         */
        @Override
        public final String toString() {
            StringBuilder buffer = new StringBuilder();

            append(buffer, "OS Arch:        ", getArch());
            append(buffer, "OS Name:        ", getName());
            append(buffer, "OS Version:     ", getVersion());
            append(buffer, "File Encoding:  ", getFileEncoding());
            append(buffer, "File Separator: ", getFileSeparator());
            append(buffer, "Line Separator: ", getLineSeparator());
            append(buffer, "Path Separator: ", getPathSeparator());

            return buffer.toString();
        }
    }

    /**
     * ǰûϢ
     */
    public static final class UserInfo {
        private final String USER_NAME = getSystemProperty("user.name", false);
        private final String USER_HOME = getSystemProperty("user.home", false);
        private final String USER_DIR = getSystemProperty("user.dir", false);
        private final String USER_LANGUAGE = getSystemProperty("user.language", false);
        private final String USER_COUNTRY = getSystemProperty("user.country", false) == null ? getSystemProperty(
                "user.region", false) : getSystemProperty("user.country", false);
        private final String JAVA_IO_TMPDIR = getSystemProperty("java.io.tmpdir", false);

        /**
         * ֹ紴˶
         */
        private UserInfo() {
        }

        /**
         * ȡõǰ¼û֣ȡϵͳԣ<code>user.name</code>
         * <p>
         * 磺<code>"admin"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getName() {
            return USER_NAME;
        }

        /**
         * ȡõǰ¼ûhomeĿ¼ȡϵͳԣ<code>user.home</code>
         * <p>
         * 磺<code>"/home/admin"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getHomeDir() {
            return USER_HOME;
        }

        /**
         * ȡõǰĿ¼ȡϵͳԣ<code>user.dir</code>
         * <p>
         * 磺<code>"/home/admin/working"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.1
         */
        public final String getCurrentDir() {
            return USER_DIR;
        }

        /**
         * ȡʱĿ¼ȡϵͳԣ<code>java.io.tmpdir</code>
         * <p>
         * 磺<code>"/tmp"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getTempDir() {
            return JAVA_IO_TMPDIR;
        }

        /**
         * ȡõǰ¼ûãȡϵͳԣ<code>user.language</code>
         * <p>
         * 磺<code>"zh"</code><code>"en"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getLanguage() {
            return USER_LANGUAGE;
        }

        /**
         * ȡõǰ¼ûĹһãȡϵͳԣJDK1.4 <code>user.country</code>JDK1.2
         * <code>user.region</code>
         * <p>
         * 磺<code>"CN"</code><code>"US"</code>
         * </p>
         * 
         * @return ֵȡãΪJavaȫƣֵڣ򷵻<code>null</code>
         * @since Java 1.2
         */
        public final String getCountry() {
            return USER_COUNTRY;
        }

        /**
         * ǰûϢתַ
         * 
         * @return ûϢַʾ
         */
        @Override
        public final String toString() {
            StringBuilder buffer = new StringBuilder();

            append(buffer, "User Name:        ", getName());
            append(buffer, "User Home Dir:    ", getHomeDir());
            append(buffer, "User Current Dir: ", getCurrentDir());
            append(buffer, "User Temp Dir:    ", getTempDir());
            append(buffer, "User Language:    ", getLanguage());
            append(buffer, "User Country:     ", getCountry());

            return buffer.toString();
        }
    }

    /**
     * ǰϢ
     */
    public static final class HostInfo {
        private final String HOST_NAME;
        private final String HOST_ADDRESS;

        /**
         * ֹ紴˶
         */
        private HostInfo() {
            String hostName;
            String hostAddress;

            try {
                InetAddress localhost = InetAddress.getLocalHost();

                hostName = localhost.getHostName();
                hostAddress = localhost.getHostAddress();
            } catch (UnknownHostException e) {
                hostName = "localhost";
                hostAddress = "127.0.0.1";
            }

            HOST_NAME = hostName;
            HOST_ADDRESS = hostAddress;
        }

        /**
         * ȡõǰơ
         * <p>
         * 磺<code>"webserver1"</code>
         * </p>
         * 
         * @return 
         */
        public final String getName() {
            return HOST_NAME;
        }

        /**
         * ȡõǰĵַ
         * <p>
         * 磺<code>"192.168.0.1"</code>
         * </p>
         * 
         * @return ַ
         */
        public final String getAddress() {
            return HOST_ADDRESS;
        }

        /**
         * ǰϢתַ
         * 
         * @return Ϣַʾ
         */
        @Override
        public final String toString() {
            StringBuilder buffer = new StringBuilder();

            append(buffer, "Host Name:    ", getName());
            append(buffer, "Host Address: ", getAddress());

            return buffer.toString();
        }
    }

    /**
     * ϵͳϢָ<code>PrintWriter</code>С
     */
    public static final void dumpSystemInfo() {
        dumpSystemInfo(new PrintWriter(System.out));
    }

    /**
     * ϵͳϢָ<code>PrintWriter</code>С
     * 
     * @param out <code>PrintWriter</code>
     */
    public static final void dumpSystemInfo(PrintWriter out) {
        out.println("--------------");
        out.println(getJvmSpecInfo());
        out.println("--------------");
        out.println(getJvmInfo());
        out.println("--------------");
        out.println(getJavaSpecInfo());
        out.println("--------------");
        out.println(getJavaInfo());
        out.println("--------------");
        out.println(getJavaRuntimeInfo());
        out.println("--------------");
        out.println(getOsInfo());
        out.println("--------------");
        out.println(getUserInfo());
        out.println("--------------");
        out.println(getHostInfo());
        out.println("--------------");
        out.flush();
    }

    /**
     * ȡϵͳԣΪJavaȫƶʧܣ򽫴<code>System.err</code>УȻ󷵻
     * <code>null</code>
     * 
     * @param name 
     * @param quiet ģʽϢ<code>System.err</code>
     * @return ֵ<code>null</code>
     */
    private static String getSystemProperty(String name, boolean quiet) {
        try {
            return System.getProperty(name);
        } catch (SecurityException e) {
            if (!quiet) {
                System.err.println("Caught a SecurityException reading the system property '" + name
                        + "'; the SystemUtil property value will default to null.");
            }

            return null;
        }
    }

    /**
     * <code>StringBuilder</code>
     * 
     * @param buffer <code>StringBuilder</code>
     * @param caption 
     * @param value ֵ
     */
    private static void append(StringBuilder buffer, String caption, String value) {
        buffer.append(caption).append(defaultIfNull(StringEscapeUtil.escapeJava(value), "[n/a]")).append("\n");
    }

    public static void main(String[] args) {
        dumpSystemInfo();

        Set<?> keys = System.getProperties().keySet();

        @SuppressWarnings("unchecked")
        List<String> list = createArrayList((Set<String>) keys);

        Collections.sort(list);

        for (String key : list) {
            String value = System.getProperty(key);

            System.out.println(key + " = " + defaultIfNull(StringEscapeUtil.escapeJava(value), "[n/a]"));
        }
    }
}
