/*
 * Copyright 2010 Alibaba Group Holding Limited.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.citrus.util.collection;

import java.util.AbstractList;
import java.util.Arrays;
import java.util.ConcurrentModificationException;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.NoSuchElementException;

/**
 * <p>
 * һHashʵ, ʵ<code>ListMap</code><code>Map</code>ӿ.
 * </p>
 * <p>
 * hashʵ־:
 * </p>
 * <ul>
 * <li>ڲķʽentry, ˳</li>
 * <li><code>DefaultHashMap</code>һ, ûнκ<code>synchronized</code></li>
 * </ul>
 * 
 * @author Michael Zhou
 * @see DefaultHashMap
 * @see ListMap
 */
public class ArrayHashMap<K, V> extends DefaultHashMap<K, V> implements ListMap<K, V> {
    private static final long serialVersionUID = 3258411729271927857L;

    // ==========================================================================
    // Ա 
    // ==========================================================================

    /** ¼entry˳. */
    protected transient DefaultHashMap.Entry<K, V>[] order;

    /** Keyбͼ. */
    private transient List<K> keyList;

    /** Valueбͼ. */
    private transient List<V> valueList;

    /** Entryбͼ. */
    private transient List<Map.Entry<K, V>> entryList;

    // ==========================================================================
    // 캯 
    // ==========================================================================

    /**
     * һյhash. ʹָĬϵĳʼ(16)Ĭϵĸϵ(0.75).
     */
    public ArrayHashMap() {
        super();
    }

    /**
     * һյhash. ʹָĳʼֵĬϵĸϵ(0.75).
     * 
     * @param initialCapacity ʼ.
     */
    public ArrayHashMap(int initialCapacity) {
        super(initialCapacity);
    }

    /**
     * һյhash. ʹָĳʼ͸ϵ.
     * 
     * @param initialCapacity ʼ
     * @param loadFactor ϵ.
     */
    public ArrayHashMap(int initialCapacity, float loadFactor) {
        super(initialCapacity, loadFactor);
    }

    /**
     * ָ<code>Map</code>ͬ<code>HashMap</code>. ʹĬϵĸϵ(0.75).
     * 
     * @param map ҪƵ<code>Map</code>
     */
    public ArrayHashMap(Map<? extends K, ? extends V> map) {
        super(map);
    }

    // ==========================================================================
    // ʵMapListMapӿڵķ 
    // ==========================================================================

    /**
     * hashаһkeyӦָvalue, 򷵻true.
     * 
     * @param value ָvalue, Ĵ.
     * @return hashаһkeyӦָvalue, 򷵻<code>true</code>.
     */
    @Override
    public boolean containsValue(Object value) {
        // Ǵ˷ǳܵĿ.  ҸЧ.
        for (int i = 0; i < size; i++) {
            Entry entry = (Entry) order[i];

            if (eq(value, entry.getValue())) {
                return true;
            }
        }

        return false;
    }

    /**
     * hashеentry.
     */
    @Override
    public void clear() {
        super.clear();
        Arrays.fill(order, null);
    }

    /**
     * ָindexvalue. indexΧ, <code>IndexOutOfBoundsException</code>.
     * 
     * @param index Ҫصvalueֵ
     * @return ָindexvalue
     */
    public V get(int index) {
        checkRange(index);
        return order[index].getValue();
    }

    /**
     * ָindexkey. indexΧ, <code>IndexOutOfBoundsException</code>.
     * 
     * @param index Ҫصkeyֵ
     * @return ָindexkey
     */
    public K getKey(int index) {
        checkRange(index);
        return order[index].getKey();
    }

    /**
     * ɾָindex. indexΧ, <code>IndexOutOfBoundsException</code>.
     * 
     * @param index Ҫɾֵ
     * @return ɾ<code>Map.Entry</code>
     */
    public Map.Entry<K, V> removeEntry(int index) {
        checkRange(index);
        return removeEntryForKey(order[index].getKey());
    }

    /**
     * key<code>List</code>.
     * 
     * @return key<code>List</code>.
     */
    public List<K> keyList() {
        return keyList != null ? keyList : (keyList = new KeyList());
    }

    /**
     * value<code>List</code>.
     * 
     * @return value<code>List</code>.
     */
    public List<V> valueList() {
        return valueList != null ? valueList : (valueList = new ValueList());
    }

    /**
     * entry<code>List</code>.
     * 
     * @return entry<code>List</code>.
     */
    public List<Map.Entry<K, V>> entryList() {
        return entryList != null ? entryList : (entryList = new EntryList());
    }

    // ==========================================================================
    // ڲ 
    // ==========================================================================

    /**
     * <code>Map.Entry</code>ʵ.
     */
    protected class Entry extends DefaultHashMap.Entry<K, V> {
        /** Entryбеֵ. */
        protected int index;

        /**
         * һµentry.
         * 
         * @param h keyhashֵ
         * @param k entrykey
         * @param v entryvalue
         * @param n еһentry
         */
        protected Entry(int h, K k, V v, DefaultHashMap.Entry<K, V> n) {
            super(h, k, v, n);
        }

        /**
         * entryɾʱ, ºentryֵ.
         */
        @Override
        protected void onRemove() {
            int numMoved = size - index;

            if (numMoved > 0) {
                System.arraycopy(order, index + 1, order, index, numMoved);
            }

            order[size] = null;

            for (int i = index; i < size; i++) {
                ((Entry) order[i]).index--;
            }
        }
    }

    /**
     * .
     */
    private abstract class ArrayHashIterator<E> implements ListIterator<E> {
        /** صentry. */
        private Entry lastReturned;

        /** ǰλ. */
        private int cursor;

        /** iteratorʱ޸ļ. */
        private int expectedModCount;

        /**
         * һlist iterator.
         * 
         * @param index ʼ
         */
        protected ArrayHashIterator(int index) {
            if (index < 0 || index > size()) {
                throw new IndexOutOfBoundsException("Index: " + index);
            }

            cursor = index;
            expectedModCount = modCount;
        }

        /**
         * ָ뵽б. (ִ֧˲)
         * 
         * @param o ҪĶ
         */
        public void add(E o) {
            throw new UnsupportedOperationException();
        }

        /**
         * ָ滻б. (<code>valueList</code>, ִ֧˲)
         * 
         * @param o Ҫ滻Ķ
         */
        public void set(E o) {
            throw new UnsupportedOperationException();
        }

        /**
         * رǷһentry.
         * 
         * @return лһentry, <code>true</code>
         */
        public boolean hasNext() {
            return cursor < size;
        }

        /**
         * رǷǰһentry.
         * 
         * @return лǰһentry, <code>true</code>
         */
        public boolean hasPrevious() {
            return cursor > 0;
        }

        /**
         * ȡһindex. һ, 򷵻<code>size</code>.
         * 
         * @return һindex
         */
        public int nextIndex() {
            return cursor;
        }

        /**
         * ȡǰһindex. ǵһ, 򷵻<code>-1</code>.
         * 
         * @return ǰһindex
         */
        public int previousIndex() {
            return cursor - 1;
        }

        /**
         * ɾһǰentry. ִǰִ<code>next()</code><code>previous()</code>.
         */
        public void remove() {
            if (lastReturned == null) {
                throw new IllegalStateException();
            }

            checkForComodification();

            removeEntryForKey(lastReturned.getKey());

            if (lastReturned.index < cursor) {
                cursor--;
            }

            lastReturned = null;
            expectedModCount = modCount;
        }

        /**
         * ȡһentry.
         * 
         * @return һentry
         */
        protected Entry nextEntry() {
            checkForComodification();

            if (cursor >= size) {
                throw new NoSuchElementException();
            }

            lastReturned = (Entry) order[cursor++];

            return lastReturned;
        }

        /**
         * ȡǰһentry.
         * 
         * @return ǰһentry
         */
        protected Entry previousEntry() {
            checkForComodification();

            if (cursor <= 0) {
                throw new NoSuchElementException();
            }

            lastReturned = (Entry) order[--cursor];

            return lastReturned;
        }

        /**
         * õǰentryֵ.
         * 
         * @param o Ҫõֵ
         */
        protected void setValue(V o) {
            if (lastReturned == null) {
                throw new IllegalStateException();
            }

            checkForComodification();

            lastReturned.setValue(o);
        }

        /**
         * Ƿͬʱ޸.
         */
        private void checkForComodification() {
            if (modCount != expectedModCount) {
                throw new ConcurrentModificationException();
            }
        }
    }

    /**
     * ȡhashkeyı.
     */
    private class KeyIterator extends ArrayHashIterator<K> {
        /**
         * һlist iterator.
         * 
         * @param index ʼ
         */
        protected KeyIterator(int index) {
            super(index);
        }

        /**
         * ȡһkey.
         * 
         * @return һkey
         */
        public K next() {
            return nextEntry().getKey();
        }

        /**
         * ȡǰһkey.
         * 
         * @return ǰһkey
         */
        public K previous() {
            return previousEntry().getKey();
        }
    }

    /**
     * ȡhashvalueı.
     */
    private class ValueIterator extends ArrayHashIterator<V> {
        /**
         * һlist iterator.
         * 
         * @param index ʼ
         */
        protected ValueIterator(int index) {
            super(index);
        }

        /**
         * ָ滻б.
         * 
         * @param o Ҫ滻Ķ(value)
         */
        @Override
        public void set(V o) {
            setValue(o);
        }

        /**
         * ȡһvalue.
         * 
         * @return һvalue
         */
        public V next() {
            return nextEntry().getValue();
        }

        /**
         * ȡǰһvalue.
         * 
         * @return ǰһvalue
         */
        public V previous() {
            return previousEntry().getValue();
        }
    }

    /**
     * ȡhashentryı.
     */
    private class EntryIterator extends ArrayHashIterator<Map.Entry<K, V>> {
        /**
         * һlist iterator.
         * 
         * @param index ʼ
         */
        protected EntryIterator(int index) {
            super(index);
        }

        /**
         * ȡһentry.
         * 
         * @return һentry
         */
        public Map.Entry<K, V> next() {
            return nextEntry();
        }

        /**
         * ȡǰһentry.
         * 
         * @return ǰһentry
         */
        public Map.Entry<K, V> previous() {
            return previousEntry();
        }
    }

    /**
     * бͼ.
     */
    private abstract class ArrayHashList<E> extends AbstractList<E> {
        /**
         * hashentryĸ.
         * 
         * @return hashеentry.
         */
        @Override
        public int size() {
            return size;
        }

        /**
         * жǷΪյhash.
         * 
         * @return Ϊ(<code>size() == 0</code>), 򷵻<code>true</code>.
         */
        @Override
        public boolean isEmpty() {
            return size == 0;
        }

        /**
         * entry.
         */
        @Override
        public void clear() {
            ArrayHashMap.this.clear();
        }

        /**
         * ȡָentry. ͬ<code>indexOf</code>.
         * 
         * @param o Ҫҵentry
         * @return ָentry
         */
        @Override
        public int lastIndexOf(Object o) {
            return indexOf(o);
        }
    }

    /**
     * entryбͼ.
     */
    private class EntryList extends ArrayHashList<Map.Entry<K, V>> {
        /**
         * жentryбǷָ.
         * 
         * @param o ҪҵĶ
         * @return entryбǷָ, 򷵻<code>true</code>
         */
        @Override
        public boolean contains(Object o) {
            if (!(o instanceof Map.Entry<?, ?>)) {
                return false;
            }

            Map.Entry<?, ?> entry = (Map.Entry<?, ?>) o;
            Entry candidate = (Entry) getEntry(entry.getKey());

            return eq(candidate, entry);
        }

        /**
         * ȡentryı.
         * 
         * @return entryı
         */
        @Override
        public Iterator<Map.Entry<K, V>> iterator() {
            return newEntryIterator();
        }

        /**
         * ɾָentry.
         * 
         * @param o Ҫɾentry
         * @return ɾɹ, <code>true</code>
         */
        @Override
        public boolean remove(Object o) {
            return removeEntry(o) != null;
        }

        /**
         * ɾָindex. indexΧ, <code>IndexOutOfBoundsException</code>.
         * 
         * @param index Ҫɾֵ
         * @return ɾ<code>Map.Entry</code>
         */
        @Override
        public Map.Entry<K, V> remove(int index) {
            checkRange(index);
            return removeEntryForKey(order[index].getKey());
        }

        /**
         * ָindexentry. indexΧ, 
         * <code>IndexOutOfBoundsException</code>.
         * 
         * @param index Ҫصentryֵ
         * @return ָindexentry
         */
        @Override
        public Map.Entry<K, V> get(int index) {
            checkRange(index);
            return order[index];
        }

        /**
         * ȡָentry.
         * 
         * @param o Ҫҵentry
         * @return ָentry
         */
        @Override
        public int indexOf(Object o) {
            if (o != null && o instanceof Map.Entry<?, ?>) {
                Entry entry = (Entry) getEntry(((Map.Entry<?, ?>) o).getKey());

                if (entry != null && entry.equals(o)) {
                    return entry.index;
                }
            }

            return -1;
        }

        /**
         * ȡlist iterator, õǰλ.
         * 
         * @param index ǰλ
         * @return list iterator
         */
        @Override
        public ListIterator<Map.Entry<K, V>> listIterator(int index) {
            return new EntryIterator(index);
        }
    }

    /**
     * keyбͼ.
     */
    private class KeyList extends ArrayHashList<K> {
        /**
         * жkeyбǷָ.
         * 
         * @param o ҪҵĶ
         * @return keyбǷָ, 򷵻<code>true</code>
         */
        @Override
        public boolean contains(Object o) {
            return ArrayHashMap.this.containsKey(o);
        }

        /**
         * ȡkeyı.
         * 
         * @return keyı
         */
        @Override
        public Iterator<K> iterator() {
            return newKeyIterator();
        }

        /**
         * ɾָkey.
         * 
         * @param o Ҫɾkey
         * @return ɾɹ, <code>true</code>
         */
        @Override
        public boolean remove(Object o) {
            Entry entry = (Entry) getEntry(o);

            if (entry == null) {
                return false;
            } else {
                removeEntry(entry);
                return true;
            }
        }

        /**
         * ɾָindex. indexΧ, <code>IndexOutOfBoundsException</code>.
         * 
         * @param index Ҫɾֵ
         * @return ɾ<code>Map.Entry</code>
         */
        @Override
        public K remove(int index) {
            checkRange(index);
            return removeEntryForKey(order[index].getKey()).getKey();
        }

        /**
         * ָindexkey. indexΧ, 
         * <code>IndexOutOfBoundsException</code>.
         * 
         * @param index Ҫصkeyֵ
         * @return ָindexkey
         */
        @Override
        public K get(int index) {
            checkRange(index);
            return order[index].getKey();
        }

        /**
         * ȡָkey.
         * 
         * @param o Ҫҵkey
         * @return ָkey
         */
        @Override
        public int indexOf(Object o) {
            Entry entry = (Entry) getEntry(o);

            if (entry != null) {
                return entry.index;
            }

            return -1;
        }

        /**
         * ȡlist iterator, õǰλ.
         * 
         * @param index ǰλ
         * @return list iterator
         */
        @Override
        public ListIterator<K> listIterator(int index) {
            return new KeyIterator(index);
        }
    }

    /**
     * valueбͼ.
     */
    private class ValueList extends ArrayHashList<V> {
        /**
         * жvalueбǷָ.
         * 
         * @param o ҪҵĶ
         * @return valueбǷָ, 򷵻<code>true</code>
         */
        @Override
        public boolean contains(Object o) {
            return ArrayHashMap.this.containsValue(o);
        }

        /**
         * ȡvalueı.
         * 
         * @return valueı
         */
        @Override
        public Iterator<V> iterator() {
            return newValueIterator();
        }

        /**
         * ɾָvalue.
         * 
         * @param o Ҫɾvalue
         * @return ɾɹ, <code>true</code>
         */
        @Override
        public boolean remove(Object o) {
            int index = indexOf(o);

            if (index != -1) {
                ArrayHashMap.this.removeEntry(index);
                return true;
            }

            return false;
        }

        /**
         * ɾָindex. indexΧ, <code>IndexOutOfBoundsException</code>.
         * 
         * @param index Ҫɾֵ
         * @return ɾ<code>Map.Entry</code>
         */
        @Override
        public V remove(int index) {
            checkRange(index);
            return removeEntryForKey(order[index].getKey()).getValue();
        }

        /**
         * ָindexvalue. indexΧ, 
         * <code>IndexOutOfBoundsException</code>.
         * 
         * @param index Ҫصvalueֵ
         * @return ָindexvalue
         */
        @Override
        public V get(int index) {
            checkRange(index);
            return order[index].getValue();
        }

        /**
         * ȡָvalue.
         * 
         * @param o Ҫҵvalue
         * @return ָvalue
         */
        @Override
        public int indexOf(Object o) {
            for (int i = 0; i < size; i++) {
                if (eq(o, order[i].getValue())) {
                    return i;
                }
            }

            return -1;
        }

        /**
         * ȡlist iterator, õǰλ.
         * 
         * @param index ǰλ
         * @return list iterator
         */
        @Override
        public ListIterator<V> listIterator(int index) {
            return new ValueIterator(index);
        }
    }

    // ==========================================================================
    // ڲ 
    // ==========================================================================

    /**
     * ʼʱhash.
     */
    @Override
    @SuppressWarnings("unchecked")
    protected void onInit() {
        order = new DefaultHashMap.Entry[threshold];
    }

    /**
     * ˷˸ķ. һentry, ͬʱentry¼orderб.
     * 
     * @param key hashkey
     * @param value hashvalue
     */
    @Override
    protected void addEntry(K key, V value) {
        int hash = hash(key);
        int i = indexFor(hash, table.length);
        Entry entry = new Entry(hash, key, value, table[i]);

        table[i] = entry;
        entry.index = size;
        order[size++] = entry;
    }

    /**
     * Ǹķ, keyı.
     * 
     * @return hashkeyı
     */
    @Override
    protected Iterator<K> newKeyIterator() {
        return new KeyIterator(0);
    }

    /**
     * Ǹķ, valueı.
     * 
     * @return hashkeyı
     */
    @Override
    protected Iterator<V> newValueIterator() {
        return new ValueIterator(0);
    }

    /**
     * Ǹķ, entryı.
     * 
     * @return hashkeyı
     */
    @Override
    protected Iterator<Map.Entry<K, V>> newEntryIterator() {
        return new EntryIterator(0);
    }

    /**
     * map. ˷entryֵʱ.
     * 
     * @param newCapacity µ
     */
    @Override
    protected void resize(int newCapacity) {
        super.resize(newCapacity);

        if (threshold > order.length) {
            @SuppressWarnings("unchecked")
            DefaultHashMap.Entry<K, V>[] newOrder = new DefaultHashMap.Entry[threshold];

            System.arraycopy(order, 0, newOrder, 0, order.length);

            order = newOrder;
        }
    }

    /**
     * <code>resize</code>ʱô˷еƵµ. Ǵ˷ǳܵĿ,
     * ΪhashԭʵַЧ.
     * 
     * @param newTable ±
     */
    @Override
    protected void transfer(DefaultHashMap.Entry<K, V>[] newTable) {
        int newCapacity = newTable.length;

        for (int i = 0; i < size; i++) {
            Entry entry = (Entry) order[i];
            int index = indexFor(entry.hash, newCapacity);

            entry.next = newTable[index];
            newTable[index] = entry;
        }
    }

    /**
     * ֵָǷԽ. , ʱ쳣.
     * 
     * @param index Ҫ쳣
     */
    private void checkRange(int index) {
        if (index >= size || index < 0) {
            throw new IndexOutOfBoundsException("Index: " + index + ", Size: " + size);
        }
    }
}
